/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import picture.*;
import picture.gui.*;

/**
 * Internal Frame zur Darstellung eines PGM-Bitmaps innerhalb eines
 * JDesktopPanes.
 * 
 * @version 1.0
 * @author Manuel Kauers
 */
public class JPictureViewer extends JInternalFrame {

    private class EventListener extends MouseAdapter 
	implements MouseMotionListener, ActionListener, ChangeListener {

	// Zum MouseMotionListener
	public void mouseDragged(MouseEvent e) {mouseMoved(e);}
	public void mouseMoved(MouseEvent e) {}

	// zur Implementierung des ChangeListener
	public void stateChanged(ChangeEvent e) {

	    // Unterrichte die Anzeigekomponente von der Aenderung und zeichne neu.
	    jimg.updateImage(); 
	    sp.setViewportView(jimg);
	}

	// zum MouseListener
	public void mousePressed(MouseEvent e) {mayBeShowPopup(e);}
	public void mouseReleased(MouseEvent e) {mayBeShowPopup(e);}

	private void mayBeShowPopup(MouseEvent e) {

	    int x = e.getX(); int y = e.getY();
	    if (e.isPopupTrigger()) {
		Point d = jimg.getLocation(); Insets s = jimg.getInsets();
		x += s.left + d.x; y += s.top + d.y;
		d = getLocation(d); s = getInsets();
		x += s.left + d.x; y += s.top + d.y + 20;
		img.getPresentation().getJPopupMenu(this).show(getDesktopPane(), x, y);
	    }
	}

	public void actionPerformed(ActionEvent e) {jimg.updateImage();}
    }

    private Picture img;
    private JPicture jimg;
    private JProgressBar pgb = new JProgressBar();
    private EventListener ev = new EventListener();
    private JScrollPane sp;
    public JLabel text = new JLabel("Reelles Bild"); 
    private JLabel zoom = new JLabel("   100%", JLabel.RIGHT);

    /**
     * Ohne Bild kein Fenster!
     */
    private JPictureViewer() {};

    /**
     * Erzeugt eine neue Bildanzeige, die img anzeigt. 
     */
    public JPictureViewer(Picture img) {this("", img);}

    /**
     * Erzeugt eine neue Bildanzeige, die img anzeigt. Mit s wird der Text in der
     * Titelleiste spezifiziert, z.B. der Dateiname. Es wird angenommen, 
     * dass img das reelle Bild repraesentiert.
     */
    public JPictureViewer(String s, Picture img) {

	super(s, true, true, true, true);

	// Daten retten und Objekte erzeugen

	this.img = img;
	jimg = new JPicture(img);
	jimg.setToolTipText("asdf"); // nicht loeschen, diese Zeile ist wesentlich!
	sp = new JScrollPane(jimg, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	JPanel status = new JPanel(new BorderLayout());

	if (img.getNumberOfLayers() == 1)
	    text.setText("Graustufenbild");
	else if (img.isComplex())
	    text.setText("komplexes Bild");
	else if (img.isColored())
	    text.setText("Farbbild");

	// Layouten

	setFrameIcon(new ImageIcon(GUIHelp.getImageURL("symbol")));
	text.setBackground(Color.lightGray);
	Container c = getContentPane();
	c.setLayout(new BorderLayout());
	setBackground(Color.lightGray);

	status.add("Center", text);
	status.add("East", zoom);

	c.add("Center", sp);
	c.add("South", pgb);
	c.add("North", status);

	// Verkabeln

	this.img.setProgressHandler(new ProgressHandler(pgb));
	this.img.addChangeListener(ev);
	jimg.addMouseListener(ev);
	jimg.addMouseMotionListener(ev);

	pack();
	show();
    }

    public double getZoomFactor() {return jimg.getZoomFactor();}

    public void setZoomFactor(double f) {

	f = Math.abs(f);
	if (f < 0.5)
	    f = 0.5;
	if (f > 4)
	    f = 4;
	
	zoom.setText(((int) (100 * f * f)) + "%");
	jimg.setZoomFactor(f);
    }

    public Picture getPicture() {return img;}
}

