/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import javax.swing.border.*;
import javax.accessibility.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.applet.*;
import java.util.*;

import picture.*;
import picture.gui.*;
import picture.filter.*;
import picture.math.*;
import picture.filetype.*;

/**
 * Hauptklasse. Zum Starten des Programms ist die main-Methode in dieser
 * Klasse auszufuehren.
 * Hinweis: Wer auf der Suche nach lesbarem, gut strukturierten Code ist,
 * wird den in den Klassen von JIPP an vielen Stellen finden. Diese Klasse
 * geh&ouml;rt diesbezueglich allerdings nicht zu den Vorzeigeobjekten...
 * 
 * @version 1.0
 * @author Manuel Kauers
 * @author Stefan Roehrich
 */
public class Main extends JFrame implements ActionListener {

    // #############################################################################
    // ###  Instanzvariablen  ######################################################
    // #############################################################################

    private Transformation letzterFilter = null;

    private JDesktopPane d;
    private int neuErzeugteBilder = 0; // wird zur Generierung neuer Dateinamen verwendet.
    private int frameZahl = 0; // Anzahl der geoeffneten Internalframes; Zur Positionsbestimmung

    private static final FFT fft = new FFT(false);
    private static final FFT fftI = new FFT(true);

    private static final FundamentalOperation mul =
	new FundamentalOperation(FundamentalOperation.PRODUCT);
    private static final FundamentalOperation div =
	new FundamentalOperation(FundamentalOperation.QUOTIENT);
    private static final FundamentalOperation add =
	new FundamentalOperation(FundamentalOperation.SUM);
    private static final FundamentalOperation sub =
	new FundamentalOperation(FundamentalOperation.DIFFERENCE);

    private static final Correlation korr = new Correlation();

    public final JLabel status1 = new JLabel("                    ");
    public final JLabel status2 = new JLabel("     ");
    public final JLabel status3 = new JLabel("                         ");

    private static Image homosatz = 
	Toolkit.getDefaultToolkit().getImage(GUIHelp.getImageURL("homomorphiesatz"));

    private JCheckBoxMenuItem mittelpunktsverschiebung = 
	new JCheckBoxMenuItem("Mittelpunktsverschiebung nach FFT", true);
    private JCheckBoxMenuItem[] toolbarItems = new JCheckBoxMenuItem[]{
	new JCheckBoxMenuItem("Symbolleiste \"Datei\"", true),
        new JCheckBoxMenuItem("Symbolleiste \"Zoom\"", true),
	new JCheckBoxMenuItem("Symbolleiste \"Filter\"", true),
	new JCheckBoxMenuItem("Symbolleiste \"Verknpfen\"", true),
	new JCheckBoxMenuItem("Symbolleiste \"Farben\"", true),
	new JCheckBoxMenuItem("Symbolleiste \"Komplex\"", true),
    };

    private JCheckBoxMenuItem lineale = new JCheckBoxMenuItem("Lineale", true);
    private RulerPane rp = new RulerPane();

    private static class ToolBarHandler implements ActionListener {

	private JCheckBoxMenuItem item;
	private JToolBar toolbar;
	private JPanel panel;
	private String direction;
	private JPanel toLayout;

	public ToolBarHandler(JCheckBoxMenuItem mi, JToolBar tb, JPanel jp, JPanel m, String d) {

	    item = mi; toolbar = tb; panel = jp; direction = d; toLayout = m;	    
	}

	public void actionPerformed(ActionEvent e) {

	    if (item.isSelected()) 
		panel.add(direction, toolbar);
	    else
		panel.remove(toolbar);

	    toLayout.revalidate();
	    toLayout.repaint();
	}
    }
    
    public Main() {

	super("JIPP");

	GUIHelp.main = this;

	setJMenuBar(holeMenue());

	d = new JDesktopPane();
	d.setOpaque(false);
	d.setBackground(Color.gray);
	rp.add("Center", d);
	lineale.addActionListener(new ActionListener() {
	    
	    public void actionPerformed(ActionEvent e) {
		if (lineale.isSelected()) rp.showRulers();
		else rp.hideRulers();
	    }
	});

	JPanel innen = new JPanel(new BorderLayout());
	JPanel oben = new JPanel(new BorderLayout());
	JPanel links = new JPanel(new BorderLayout());
	innen.add("North", oben); innen.add("West", links); innen.add("Center", rp);

	for (int i = 0; i < 4; i++) {
	    JPanel jp = new JPanel(new BorderLayout());
	    JToolBar tb = holeToolbar(i);
	    jp.add("North", tb); oben.add("West", jp);
	    toolbarItems[i].addActionListener(new ToolBarHandler(toolbarItems[i], tb, jp, innen,
								 "North"));
	    jp = new JPanel(new BorderLayout(0, 0)); oben.add("Center", jp); oben = jp;
	}

	for (int i = 4; i < 6; i++) {
	    JPanel jp = new JPanel(new BorderLayout());
	    JToolBar tb = holeToolbar(i); tb.setOrientation(tb.VERTICAL);
	    jp.add("East", tb); links.add("North", jp);
	    toolbarItems[i].addActionListener(new ToolBarHandler(toolbarItems[i], tb, jp, innen,
								 "East"));
	    jp = new JPanel(new BorderLayout(0, 0)); links.add("East", jp); links = jp;
	}

	JPanel statusleiste = new JPanel(new GridLayout(1, 0));
	status1.setBorder(BorderFactory.createLoweredBevelBorder());
	statusleiste.add("East", status1); 
	status2.setBorder(BorderFactory.createLoweredBevelBorder());
	statusleiste.add("Center", status2); 
	status3.setBorder(BorderFactory.createLoweredBevelBorder());
	statusleiste.add("West", status3); 
	
	Container c = getContentPane();
	c.add("Center", innen);
	c.add("South", statusleiste);

	Toolkit tk = Toolkit.getDefaultToolkit();

        addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});
	setIconImage(tk.getImage(GUIHelp.getImageURL("symbol")));

	setSize(540, 400);	
	//	Dimension d = tk.getScreenSize();
	//	setLocation(d.width / 2 - 320, d.height / 2 - 240);

	// Standarddateiformate
	picture.filetype.JPFPictureType.init();
	picture.filetype.JavaImagePictureType.init();
	picture.filetype.PGMPictureType.init();
    }

    // #############################################################################
    // ###  Dateioperationen: Lesen und Schreiben  #################################
    // #############################################################################

    public static File openFile(Component parent) {
	
	return GUIHelp.openFile(parent, ".");
    }

    public static Picture openPicture(Component parent, String selected) 
	throws PictureException, IOException {

	return PictureTypeFactory.loadPicture(GUIHelp.openFile(parent, selected));
    }

    public static Picture openPicture(Component parent) 
	throws PictureException, IOException {

	File f = openFile(parent);
	if (f == null)
	    return null;
	return PictureTypeFactory.loadPicture(f);
    }

    private File openFile() throws FileNotFoundException, PictureException {

	return openFile(this);
    }

    // #############################################################################
    // ###  eigene Bildumwandlungsmehtoden  ########################################
    // #############################################################################

    private static final Picture farbToGrau(Picture p) {

	if (!p.isColored() || p.getNumberOfLayers() != 3) return p;

	double[][] l = new double[p.getWidth()][p.getHeight()];

	Layer red = p.getLayer(0); double r; 
	double rMax = red.getMaximum(); double rMin = red.getMinimum();
	Layer green = p.getLayer(1); double g;
	double gMax = green.getMaximum(); double gMin = green.getMinimum();
	Layer blue = p.getLayer(2); double b;
	double bMax = blue.getMaximum(); double bMin = blue.getMinimum();

	double rFaktor = 0.299 / (rMax - rMin);
	double gFaktor = 0.587 / (gMax - gMin);
	double bFaktor = 0.114 / (bMax - bMin);
	
	for (int x = 0; x < p.getWidth(); x++)
	    for (int y = 0; y < p.getHeight(); y++) {
		
		r = red.getValue(x, y);
		g = green.getValue(x, y);
		b = blue.getValue(x, y);
		
		l[x][y] = (r - rMin) * rFaktor + (g - gMin) * gFaktor + (b - bMin) * bFaktor;
	    }
	
	p = new Picture(p, new Layer[]{new Layer(l)});
	p.setDefaultPresentation();
	return p;
    }


    // #############################################################################
    // ###  Methoden, die vom Menue aus angesprochen werden  #######################
    // #############################################################################

    private void oeffnen() throws FileNotFoundException, PictureException {

	File theFile = openFile();

	try {
	    if (theFile != null) 
		addPictureViewer(theFile.getName(), PictureTypeFactory.loadPicture(theFile));
	} catch (IOException e) {
	    if (GUIHelp.debug) 
		e.printStackTrace();
	    throw new FileNotFoundException();
	}
    }

    private void speichern() throws Exception {
	Picture p = getSelectedPicture();
	GUIHelp.saveTo(this, p, new File(p.getFileName()));
    }

    private void drucken() {

	Picture p;
	try {
	    p = getSelectedPicture();
	} catch (FileNotFoundException e) {
	    GUIHelp.error(e, "Kein Bild ausgewhlt");
	    return;
	}
	PrintJob job = Toolkit.getDefaultToolkit().getPrintJob(this, p.getFileName(), null);
	if (job != null) {
	    Graphics g = job.getGraphics();
	    if (g != null) {
		Dimension d = job.getPageDimension(); 
		int abstand = 10;
		int y = d.height / 2; 
		int x = d.width / 2;
		Image img = p.getImage();
		if (img.getWidth(this) > d.width - 2 * abstand)
		    x = abstand;
		else 
		    x -= img.getWidth(this) / 2;
		if (img.getHeight(this) > d.height - 2 * abstand)
		    y = abstand;
		else
		    y -= img.getHeight(this) / 2;
		g.drawImage(img, x, y, this);		
		g.dispose();
		job.end();
	    }
	}
    }

    private void fft() throws FileNotFoundException, PictureException {

	Picture p = getSelectedPicture();
	p.getProgressHandler().setBounds(0, 2 * Math.max(p.getWidth(), p.getHeight()));
	fft.setShifting(mittelpunktsverschiebung.isSelected());
	addPictureViewer(fft.of(p));
	letzterFilter = fft;
    }

    private void inverseFft() throws FileNotFoundException, PictureException {

	Picture p = getSelectedPicture();
	p.getProgressHandler().setBounds(0, 2 * Math.max(p.getWidth(), p.getHeight()));
	fftI.setShifting(mittelpunktsverschiebung.isSelected());
	addPictureViewer(fftI.of(p));
	letzterFilter = fftI;
    }

    private void shiften() throws Exception {

	ShiftPane sp = new ShiftPane(this, getSelectedPicture());
	sp.show();
	if (sp.getClosingCode() == sp.OK) {
	    addPictureViewer(sp.getResultingPicture());
	    letzterFilter = null;
	}	
    }

    /**
     * op = 0  =>  Korrelation
     * op = 1  =>  Summe
     * op = 2  =>  Differenz
     * op = 3  =>  Produkt
     * op = 4  =>  Quotient
     */
    private void verknuepfenMit(int op) throws PictureException {

	try {
	    Picture p = getSelectedPicture();
	    Picture q = openPicture(this);
	    if (q == null) return;
	    
	    switch (op) {
	    case 0: 
		p.getProgressHandler().setBounds(0, 6 * p.getWidth());
		q = korr.of(p, q); break;
	    case 1: q = add.of(p, q); break;
	    case 2: q = sub.of(p, q); break;
	    case 3: q = mul.of(p, q); break;
	    case 4: q = div.of(p, q); break;
	    } 
	    
	    addPictureViewer(q);
	    letzterFilter = null;

	} catch (IOException e) {
	    throw new PictureException("Fehlerhafte Datei");
	}
    }

    private void korrellierenMit() throws Exception {verknuepfenMit(0);}
    private void plus() throws Exception {verknuepfenMit(1);}
    private void minus() throws Exception {verknuepfenMit(2);}
    private void mal() throws Exception {verknuepfenMit(3);}
    private void durch() throws Exception {verknuepfenMit(4);}

    /**
     * i = 0: reelles Bild
     * i = 1: imaginaeres Bild
     * i = 2: Amplitudenbild
     * i = 3: Phasenbild
     */
    private void andereAnsicht(int i) throws FileNotFoundException, PictureException {

	Picture p = getSelectedPicture();
	
	if (!p.isComplex())
	    throw new PictureException("Das Bild enthlt keine komplexen Daten.");

	Layer[] layers = null;
	switch (i) {
	case 0: layers = new Layer[]{p.getLayer(0)}; break;
	case 1: layers = new Layer[]{p.getLayer(1)}; break;
	case 2: {// Amplitudenbild
	    double[][] l = new double[p.getWidth()][p.getHeight()];
	    for (int x = 0; x < p.getWidth(); x++)
		for (int y = 0; y < p.getHeight(); y++)
		    l[x][y] = new Complex(p.getValue(x, y)).getAbsolutValue();
	    layers = new Layer[]{new Layer(l)};
	}; break;
	case 3: {// Phasenbild
	    double[][] l = new double[p.getWidth()][p.getHeight()];
	    for (int x = 0; x < p.getWidth(); x++)
		for (int y = 0; y < p.getHeight(); y++)
		    l[x][y] = new Complex(p.getValue(x, y)).getPolarAngle();
	    layers = new Layer[]{new Layer(l)};
	}; break;
	default: return;
	}

	p = new Picture(p, layers);
	p.setDefaultPresentation();
	addPictureViewer(p);
    }

    private void info() {

	String s = "JIPP  -  Java Image Processing Program\n\n" + 
	    "(c) 2000, 2001 by\n" +
	    "Stefan Rhrich (http://www.roehri.ch/~sr/)\n" +
	    "und\n" +
	    "Manuel Kauers (http://www.kauers.de/)";
	JOptionPane.showMessageDialog(this, s, "Info", JOptionPane.INFORMATION_MESSAGE);
    }

    private void groesser() throws FileNotFoundException {

	JPictureViewer jpv = getSelectedPictureViewer();
	jpv.setZoomFactor(Math.sqrt(2) * jpv.getZoomFactor());
    }

    private void kleiner() throws FileNotFoundException {

	JPictureViewer jpv = getSelectedPictureViewer();
	jpv.setZoomFactor(jpv.getZoomFactor() / Math.sqrt(2));
    }

    private void normalGroesse() throws FileNotFoundException {

	JPictureViewer jpv = getSelectedPictureViewer();
	jpv.setZoomFactor(1);
    }

    private void homosatz() {

	JPanel hsp = new JPanel(new GridLayout(1, 1, 1, 1));
	hsp.add(new JLabel(new ImageIcon(homosatz)));
	hsp.setBorder(BorderFactory.createEtchedBorder());

	JOptionPane.showMessageDialog(this, hsp, "Homomorphiesatz", JOptionPane.PLAIN_MESSAGE);
	JOptionPane.showMessageDialog(this, 
				      "Das kriegen Sie nie wieder so erklrt!",
				      "Homomorphiesatz",
				      JOptionPane.WARNING_MESSAGE,
				      new ImageIcon(GUIHelp.getImageURL("baustelle")));
    }

    private void schliessen(boolean b) {

	if (b) 
	    // aktuelles Fenster schliessen
	    try {
		getSelectedPictureViewer().doDefaultCloseAction();
	    } catch (FileNotFoundException ign) {
		if (GUIHelp.debug)
		    ign.printStackTrace();
	    }
	else {
	    // alle Fenster schliessen
	    JInternalFrame[] jifs = d.getAllFrames();
	    for (int i = 0; i < jifs.length; i++) 
		jifs[i].doDefaultCloseAction();
	}
    }

    public void waveletKont() {

	Picture p = null;
	try {
	    p = getSelectedPicture();
	} catch (Exception gehtAuchOhne) {}
	WaveletCenter wc = new WaveletCenter(this, p);
	wc.show();
	if (wc.getClosingCode() == DialogPane.OK) {
	    addPictureViewer(wc.getResultingPicture());
	    letzterFilter = null;
	}
    }

    private void transform(Transformation t) throws FileNotFoundException, PictureException {

	Picture p = getSelectedPicture();
	DialogPane dp = t.filterOptionPane(this, p);
	if (dp != null) {
	    dp.show();
	    if (dp.getClosingCode() == dp.OK) {
		addPictureViewer(dp.getResultingPicture());
		letzterFilter = dp.getTransformation();
	    }
	} else {
	    addPictureViewer(t.of(p));
	    letzterFilter = t;
	}
    }

    private void customFilter() throws FileNotFoundException, PictureException {

	Transformation t = null;
	String message = "Bitte Klassennamen des anzuwendenen Filters eingeben:";
	String fn = JOptionPane.showInputDialog(this, message, "Klassenname", 
						JOptionPane.QUESTION_MESSAGE);

	if(fn != null) {
	    Picture p = getSelectedPicture();
	    try {
		t = (Transformation)Class.forName(fn).newInstance();
	    } catch(Exception e) {
		GUIHelp.error(e);
		return;
	    }
	    DialogPane dp = t.filterOptionPane(this, p);
	    if(dp != null) {
		dp.show();
		if (dp.getClosingCode() == dp.OK) {
		    addPictureViewer(dp.getResultingPicture());
		    letzterFilter = dp.getTransformation();
		}
	    } else {
		addPictureViewer(t.of(p));
		letzterFilter = t;
	    }
	}
    }

    private void saveLastFilter() {

	if (letzterFilter == null || !(letzterFilter instanceof DefaultTransformation)) {
	    GUIHelp.error("Es wurde noch kein Filter angewandt\n" + 
			   "oder der letzte Filter untersttzt\n" +
			   "kein Speichern.");
	    return;
	}
	    
	DefaultTransformation dt = (DefaultTransformation) letzterFilter;
	GUIHelp.saveFilter(this, dt, new File(dt.getPreferredFileName()));
    }

    private void filterFromFile() throws Exception {

	Transformation t = DefaultTransformation.loadFrom(GUIHelp.openFilter(this, "."));
	if (t == null) 
	    return;
	    
	transform(t);
    }

    /*
      i = 1  => heller
      i = 2  => dunkler
      i = 3  => invertierne
     */
    private void graufilter(int i) throws Exception {

	Picture p = getSelectedPicture();

	Term t = new Term("-x");
	switch (i) {
	case 1: t = new Term("x^(1/3)"); break;
	case 2: t = new Term("x^3"); break;
	}

	// FIXME: 255 statt nicht mehr vorhandener Anzahl Graustufen.
	addPictureViewer(new GrayScale(t, 255).of(p));
    }

    private void bildInfo() throws Exception {

	GUIHelp.info(getSelectedPicture());
    }

    private void abstand() throws Exception {

	Picture p = getSelectedPicture();
	Picture q = null;

	try {
	    q = openPicture(this);
	} catch (IOException e) {
	    throw new PictureException("Fehlerhafte Datei");
	}

	if (q == null) return;
	    
	double d = DefaultTransformation.getDistance(p, q);
	String text = "Mittlerer quadratischer Abstand\nder angegebenen Bilder: " + d;

	JOptionPane.showMessageDialog(this, text, "JIPP", JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * i = 0: nur Rot 
     * i = 1: nur Grn
     * i = 2: nur Blau
     * i = 3: kein Rot
     * i = 4: kein Grn
     * i = 5: kein Blau
     */
    private void farbebenen(int i) throws PictureException, FileNotFoundException {

	Picture p = getSelectedPicture();

	if (!p.isColored())
	    throw new PictureException("Das Bild ist kein Farbbild.");

	Layer[] layers = new Layer[3];
	for (int j = 0; j < 3; j++)
	    layers[j] = p.getLayer(j);

	Layer leer = new Layer(new double[p.getWidth()][p.getHeight()]);
	
	switch (i) {
	case 0: layers[1] = leer; layers[2] = leer; break;
	case 1: layers[0] = leer; layers[2] = leer; break;
	case 2: layers[0] = leer; layers[1] = leer; break;
	case 3: layers[0] = leer; break;
	case 4: layers[1] = leer; break;
	case 5: layers[2] = leer; break;
	default: return;
	}

	addPictureViewer(new Picture(p, layers));
    }

    /**
     * i = 0: to Schwarz-Wei
     * i = 1: to Grau
     * i = 2: to Farbbild
     */
    private void farbtiefe(int i) throws PictureException, FileNotFoundException {

	Picture p = getSelectedPicture();

	switch (i) {
	case 0: {
	    if (p.isColored()) 
		p = farbToGrau(p);
	    else 
		p = new Identity().of(p);
	    p.setPresentation(PresentationFactory.presentationByName(p, "Schwarz-Wei-Darstellung"));
	}; break;
	case 1: { 
	    if (p.isColored()) 
		p = farbToGrau(p);
	    else
		p = new Identity().of(p);
	    p.setDefaultPresentation();
	}; break;
	case 2: {
	    if (p.isColored()) {
		p = new Identity().of(p);
		break;
	    }
	    if (p.getNumberOfLayers() != 1) 
		throw new PictureException("Ungltiges Datenformat");
		
	    Layer pic = p.getLayer(0);
	    p = new Picture(p, new Layer[]{pic, pic, pic});
	    p.setPresentation(PresentationFactory.presentationByName(p, "Farbdarstellung"));
	}; break;
	default: return;
	}

	addPictureViewer(p);
    }

    private void redundanteZerlegung() throws PictureException, FileNotFoundException {

	addPictureViewer(Wavelet.getHaarWavelet(0).getRedundantAnalysis(getSelectedPicture()));
    }

    private void redundanteSynthese() throws PictureException, FileNotFoundException {

	addPictureViewer(Wavelet.getHaarWavelet(0).getRedundantSynthesis(getSelectedPicture()));
    }

    private void kltFarben() throws PictureException, FileNotFoundException {

	ColorKLTPane ckp = new ColorKLTPane(this, getSelectedPicture());
	ckp.show();
	if (ckp.getClosingCode() == ckp.OK) 
	    addPictureViewer(ckp.getResultingPicture());       
    }

    private void kltWavelet() throws PictureException, FileNotFoundException {

	WaveletKLTPane wkp = new WaveletKLTPane(this, getSelectedPicture());
	wkp.show();
	if (wkp.getClosingCode() == wkp.OK)
	    addPictureViewer(wkp.getResultingPicture());
    }

    // #############################################################################
    // ###  EventListenerMethoden  #################################################
    // #############################################################################

    /**
     * Klappert die Menueeintraege und die Toolbareintraege ab.
     */
    public void actionPerformed(final ActionEvent e) {

	new Thread() {
	  
	    public void run() {

		String k = e.getActionCommand(); 
		try {
		    if (k.equals("ffnen...")) oeffnen();
		    else if (k.equals("Speichern")) speichern();
		    else if (k.equals("Drucken...")) drucken();
		    else if (k.equals("Schlieen")) schliessen(true);
		    else if (k.equals("alles Schlieen")) schliessen(false);
		    else if (k.equals("Beenden")) System.exit(0);
		    else if (k.equals("FFT")) fft();
		    else if (k.equals("inverse FFT")) inverseFft();
		    else if (k.equals("filtern...")) 
			transform(new PassFilter(PassFilter.BANDPASSFILTER));
		    else if (k.equals("skalieren...")) transform(new GrayScale(0, 255));
		    else if (k.equals("mal...")) mal();
		    else if (k.equals("plus...")) plus();
		    else if (k.equals("minus...")) minus();
		    else if (k.equals("durch...")) durch();
		    else if (k.equals("korellieren mit...")) korrellierenMit();
		    else if (k.equals("Realteilbild")) andereAnsicht(0);
		    else if (k.equals("Imaginrteilbild")) andereAnsicht(1);
		    else if (k.equals("Amplitudenbild")) andereAnsicht(2);
		    else if (k.equals("Phasenbild")) andereAnsicht(3);
		    else if (k.equals("Info")) info();
		    else if (k.equals("vergrern")) groesser();
		    else if (k.equals("verkleinern")) kleiner();
		    else if (k.equals("Normalgre")) normalGroesse();
		    else if (k.equals("Homomorphiesatz")) homosatz();
		    else if (k.equals("zyklisch verschieben...")) shiften();
		    else if (k.equals("diskret (MRA)")) 
			transform(Wavelet.getHaarWavelet(Wavelet.ANALYSIS));
		    else if (k.equals("kontinuierlich")) waveletKont();
		    else if (k.equals("Matrixfilter...")) transform(new MatrixFilter());
		    else if (k.equals("Morphologische Filter..."))
			transform(new MorphFilter(MorphFilter.MEDIAN));
		    else if (k.equals("Eigener Filter...")) customFilter();
		    else if (k.equals("90 Grad")) transform(new Rotation(1));
		    else if (k.equals("180 Grad")) transform(new Rotation(2));
		    else if (k.equals("270 Grad")) transform(new Rotation(3));
		    else if (k.equals("kopieren")) transform(new Identity());
		    else if (k.equals("horizontal")) 
			transform(new Reflection(Reflection.HORIZONTAL));
		    else if (k.equals("vertikal"))
			transform(new Reflection(Reflection.VERTICAL));
		    else if (k.equals("Filter aus Datei...")) filterFromFile();
		    else if (k.equals("letzten Filter speichern...")) saveLastFilter();
		    else if (k.equals("aufhellen")) graufilter(1);
		    else if (k.equals("verdunkeln")) graufilter(2);
		    else if (k.equals("invertieren")) graufilter(3);
		    else if (k.equals("Bilddaten...")) bildInfo();
		    else if (k.equals("Abstand zu...")) abstand();
		    else if (k.equals("nur Rotanteil")) farbebenen(0);
		    else if (k.equals("nur Grnanteil")) farbebenen(1);
		    else if (k.equals("nur Blauanteil")) farbebenen(2);
		    else if (k.equals("Rotanteil ausblenden")) farbebenen(3);
		    else if (k.equals("Grnanteil ausblenden")) farbebenen(4);
		    else if (k.equals("Blauanteil ausblenden")) farbebenen(5);
		    else if (k.equals("Schwarz-Wei-Bild")) farbtiefe(0);
		    else if (k.equals("Graustufenbild")) farbtiefe(1);
		    else if (k.equals("Farbbild")) farbtiefe(2);
		    else if (k.equals("redundante Haar-Analyse")) redundanteZerlegung();
		    else if (k.equals("redundante Haar-Synthese")) redundanteSynthese();
		    else if (k.equals("auf dem Farbraum...")) kltFarben();
		    else if (k.equals("auf Details der Waveletzerlegung...")) kltWavelet();
		    else GUIHelp.error("Not yet implemented: " + k);
		} catch (Exception ex) {
		    GUIHelp.error(ex);
		}
	    }
	}.start();
    }

    // #############################################################################
    // ###  Tools  #################################################################
    // #############################################################################

    /*
     * 0: Datei
     * 1: Zoom
     * 2: Filter
     * 3: Verknpfen
     * 4: Farben
     * 5: Komplex
     */
    private JToolBar holeToolbar(int i) {

	JToolBar tb = new JToolBar();
	tb.setFloatable(false);
	
	switch (i) {
	case 0: 
	    addTool(tb, "oeffnen", "ffnen", "ffnen...");
	    addTool(tb, "speichern", "Speichern", "Speichern");
	    addTool(tb, "drucken", "Drucken", "Drucken...");
	    addTool(tb, "schliessen", "Schlieen", "Schlieen");
	    addTool(tb, "copy", "Duplizieren", "kopieren");
	    break;
	case 5:
	    addTool(tb, "phase", "Phasenbild", "Phasenbild");
	    addTool(tb, "betrag", "Amplitudenbild", "Amplitudenbild");
	    addTool(tb, "realteil", "Realteilbild", "reelles Bild");
	    addTool(tb, "imaginaerteil", "Imaginrteilbild", "imaginres Bild");
	    break;
	case 4:
	    addTool(tb, "rot", "nur Rotanteil", "nur Rotanteil");
	    addTool(tb, "gruen", "nur Grnanteil", "nur Grnanteil");
	    addTool(tb, "blau", "nur Blauanteil", "nur Blauanteil");
	    addTool(tb, "rotgruen", "Rot- und Grnanteil", "Blauanteil ausblenden");
	    addTool(tb, "rotblau", "Rot- und Blauanteil", "Grnanteil ausblenden");
	    addTool(tb, "gruenblau", "Grn- und Blauanteil", "Rotanteil ausblenden");
	    break;
	case 3:
	    addTool(tb, "plus", "Addieren", "plus...");
	    addTool(tb, "minus", "Subtrahieren", "minus...");
	    addTool(tb, "mal", "Multiplizieren", "mal...");
	    addTool(tb, "geteilt", "Dividieren", "durch...");
	    break;
	case 2:
	    addTool(tb, "skalierung", "skalieren", "skalieren...");
	    addTool(tb, "filter", "Bandfilter", "filtern...");
	    addTool(tb, "wavelets", "Wavelettransformation", "diskret (MRA)");
	    addTool(tb, "matrix", "Matrixfilter", "Matrixfilter...");
	    addTool(tb, "morph", "Morphologische Filter", "Morphologische Filter...");
	    break;
	case 1:
	    addTool(tb, "zoomin", "vergrern", "vergrern");
	    addTool(tb, "zoomout", "verkleinern", "verkleinern");
	    addTool(tb, "preferredsize", "Normalgre", "Normalgre");
	    break;
	}
	return tb;
    }

    private void addTool(JToolBar tb, String dnam, String tipText, String command) {

	JButton b = new JButton(new ImageIcon(GUIHelp.getImageURL(dnam)));
	b.addActionListener(this);
	b.setMargin(new Insets(0, 0, 0, 0));
	b.setToolTipText(tipText);
	b.setActionCommand(command);
	b.setFocusPainted(false);
	tb.add(b);
    }

    private JMenuBar holeMenue() {
	
	JMenuBar mb = new JMenuBar();
	JMenu m; JMenu m0;

	m = new JMenu("Datei"); mb.add(m);

	addMenuItem(m, "ffnen...");
	addMenuItem(m, "Speichern");
	addMenuItem(m, "Drucken...");
	addMenuItem(m, "Schlieen");

	m.addSeparator();

	addMenuItem(m, "alles Schlieen");
	addMenuItem(m, "Beenden");

	m = new JMenu("Bild"); mb.add(m);

	m0 = new JMenu("komplexes Bild"); m.add(m0);

	addMenuItem(m0, "Realteilbild");
	addMenuItem(m0, "Imaginrteilbild");
	addMenuItem(m0, "Amplitudenbild");
	addMenuItem(m0, "Phasenbild");

	m0 = new JMenu("Farbebenen"); m.add(m0);

	addMenuItem(m0, "nur Rotanteil");
	addMenuItem(m0, "nur Grnanteil");
	addMenuItem(m0, "nur Blauanteil");
	addMenuItem(m0, "Rotanteil ausblenden");
	addMenuItem(m0, "Grnanteil ausblenden");
	addMenuItem(m0, "Blauanteil ausblenden");

	m0 = new JMenu("Farbtiefe"); m.add(m0);

	addMenuItem(m0, "Schwarz-Wei-Bild");
	addMenuItem(m0, "Graustufenbild");
	addMenuItem(m0, "Farbbild");

	m0 = new JMenu("verknpfen"); m.add(m0);

	addMenuItem(m0, "plus...");
	addMenuItem(m0, "minus...");
	addMenuItem(m0, "mal...");
	addMenuItem(m0, "durch...");

	m.addSeparator();

	addMenuItem(m, "Abstand zu...");
	addMenuItem(m, "Bilddaten...");
	
	m = new JMenu("Filter"); mb.add(m);

	m0 = new JMenu("Fouriertransformation"); m.add(m0);

	addMenuItem(m0, "FFT");
	addMenuItem(m0, "inverse FFT");
	m0.addSeparator();
	addMenuItem(m0, "filtern...");
	addMenuItem(m0, "korellieren mit...");

	m0 = new JMenu("Wavelettransformation"); m.add(m0);

	addMenuItem(m0, "diskret (MRA)");
	addMenuItem(m0, "kontinuierlich");
	m0.addSeparator();
	addMenuItem(m0, "redundante Haar-Analyse");
	addMenuItem(m0, "redundante Haar-Synthese");

	m0 = new JMenu("Karhunen-Loeve-Transformation"); m.add(m0);

	addMenuItem(m0, "auf dem Farbraum...");
	addMenuItem(m0, "auf Details der Waveletzerlegung...");

	m0 = new JMenu("Grauwertskalierung"); m.add(m0);

	addMenuItem(m0, "aufhellen");
	addMenuItem(m0, "verdunkeln");
	addMenuItem(m0, "invertieren");
	addMenuItem(m0, "skalieren...");

	m0 = new JMenu("Drehen um"); m.add(m0);

	addMenuItem(m0, "90 Grad");
	addMenuItem(m0, "180 Grad");
	addMenuItem(m0, "270 Grad");

	m0 = new JMenu("Spiegeln"); m.add(m0);
	
	addMenuItem(m0, "horizontal");
	addMenuItem(m0, "vertikal");

	addMenuItem(m, "Morphologische Filter...");
	addMenuItem(m, "Matrixfilter...");
	addMenuItem(m, "zyklisch verschieben...");

	m.addSeparator();
	
	addMenuItem(m, "Eigener Filter...");
	addMenuItem(m, "Filter aus Datei...");
	addMenuItem(m, "letzten Filter speichern...");

	m = new JMenu("Ansicht"); mb.add(m);
	for (int i = 0; i < toolbarItems.length; i++)
	    m.add(toolbarItems[i]);

	m.addSeparator();

	m.add(lineale);

	m = new JMenu("Optionen"); mb.add(m);
	m.add(mittelpunktsverschiebung);

	m = new JMenu("Hilfe"); mb.add(m);
	addMenuItem(m, "Homomorphiesatz");
	addMenuItem(m, "Info");

	return mb;
    }

    private void addMenuItem(JMenu m, String s) {

	JMenuItem mi = new JMenuItem(s);
	m.add(mi);
	mi.addActionListener(this);
    }

    private Picture getSelectedPicture() throws FileNotFoundException {

	return getSelectedPictureViewer().getPicture();
    }

    private JPictureViewer getSelectedPictureViewer() throws FileNotFoundException {

	JInternalFrame [] jifs = d.getAllFrames();
	if (jifs.length == 0)
	    throw new FileNotFoundException("Es ist kein Bild geffnet");

	return (JPictureViewer) jifs[0];
    }

    public void addPictureViewer(Picture p) {

	String name = "neu" + (++neuErzeugteBilder);
	name += "." + p.getPictureType().getExtension();
	addPictureViewer(name, p); p.setFileName(name);
    }

    private static boolean istZweierpotenz(int i) {

	if (i == 0) return false;
	if (i < 0) i = -i;
	
	for (; i > 1; i /= 2)
	    if (i % 2 == 1) return false;

	return true;
    }

    /**
     * Falls das Bild keine Zweierpotenz ist, wird gefragt, ob es eigebettet werden
     * soll.
     */
    public void addPictureViewer(String title, Picture p) {

	int w = p.getWidth();
	int h = p.getHeight();

	if (w != h || !istZweierpotenz(w)) {
	    
	    String text = "Viele Funktionen in diesem Programm bentigen\n" +
		"quadratische Bilder, deren Kantenlnge eine Potenz\n" +
		"von zwei ist. Bei diesem Bild ist dies nicht der Fall.\n" +
		"Soll das Bild in ein quadratisches Bild, dessen Kanten-\n" +
		"lnge eine Zweierpotenz ist, eingebettet werden?";
	    switch (JOptionPane.showConfirmDialog(this, text, "JIPP",
						  JOptionPane.YES_NO_CANCEL_OPTION,
						  JOptionPane.QUESTION_MESSAGE)) {
	    case JOptionPane.YES_OPTION: 
		p = p.getEmbeddedInstance();
		break;
	    case JOptionPane.NO_OPTION: break;
	    default: return;
	    }
	}

	JPictureViewer jpv = new JPictureViewer(title, p);
	jpv.setLocation(30 * (frameZahl % 5), 30 * (frameZahl++ % 5));
	
	d.add(jpv, new Integer(1));
	d.moveToFront(jpv);
    }

    // #############################################################################
    // ###  Hauptprogramm  #########################################################
    // #############################################################################

    public static void main(String[] args) {

        Main f = new Main();
	boolean info = true;

	for (int i = 0; i < args.length; i++) 
	    try {
		String opt = args[i].toLowerCase();
		if (opt.equals("-win")) {
		    String win = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
		    UIManager.setLookAndFeel(win);
		    SwingUtilities.updateComponentTreeUI(f);
		} else if (opt.equals("-debug")) {
		    GUIHelp.debug = true;
		} else if (opt.equals("-help")) {
		    System.out.println("JIPP -- Java Image Processing Programm");
		    System.out.println("(c) 2000 by Manuel Kauers und Stefan Rhrich");
		    System.exit(0);
		} else if (opt.equals("-noinfo")) {
		    info = false;
		} else if (opt.startsWith("-open:")) {
		    String dnam = opt.substring(6);
		    f.addPictureViewer(dnam, PictureTypeFactory.loadPicture(dnam));
		} else if (opt.startsWith("-type:")) {
		    Class.forName(args[i].substring(6));
		} else
		    System.err.println("unbekannte Option: " + args[i]);
	    } catch (Exception exc) {
		System.err.println("Option " + args[i] + " konnte nicht verarbeitet werden");
		if (GUIHelp.debug)
		    System.err.println("  " + exc.getClass().getName() + ":" + exc.getMessage());
	    }

	f.setVisible(true);
	if (info)
	    f.info();
    }
}
