/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

/**
 * Implementiert eine zweistellige Operation nach Vorbild von <tt>Transformation</tt>.
 */
public abstract class Combination {

    public Combination() {}

    public abstract Layer of(ProgressHandler ph, Layer a, Layer b) throws PictureException;

    public Layer of(Layer a, Layer b) throws PictureException {return of(null, a, b);}

    public Picture of(Picture a, Picture b) 
	throws PictureException {

	if (a.getWidth() != b.getWidth() || a.getHeight() != b.getHeight() ||
	    a.getNumberOfLayers() != b.getNumberOfLayers())
	    
	    throw new PictureException("Die beiden Bilder haben nicht die gleiche Gre\n" +
				       "oder eine unterschiedliche Anzahl an Layern");

	Layer[] l = new Layer[a.getNumberOfLayers()];

	for (int i = 0; i < a.getNumberOfLayers(); i++)
	    l[i] = of(a.getLayer(i), b.getLayer(i));

	return new Picture(a, l);
    }

    /**
     * Erzeugt ein Layer von der selben Instanz wie <tt>p</tt> mit dem Inhalt von
     * <tt>data</tt>
     * @param p: Vorlage
     * @param data: Inhalt des neuen Bildes
     * @return ein Layer wie <tt>p</tt>, mit <tt>data</tt> als Inhalt.
     */
    protected Layer createNewInstance(Layer p, double[][] data) {

	return p.createNewInstance("", data);
    }
}
