/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;

public class ComplexPresentation implements PresentationType {

    private final PresentationType pt = this;

    private final class MyPresentation extends ColorModel implements Presentation {

	private final JRadioButtonMenuItem reMenuItem = 
	    new JRadioButtonMenuItem("Realteil", true);
	private final JRadioButtonMenuItem imMenuItem = 
	    new JRadioButtonMenuItem("Imaginrteil", false);
	private final Picture p;

	public MyPresentation(Picture p) {

	    super(31); this.p = p;
	    ButtonGroup b = new ButtonGroup();
	    b.add(reMenuItem); b.add(imMenuItem);
	}

	public MyPresentation(Picture p, String s) throws IllegalArgumentException {

	    this(p);
	    if (s.length() < 2) throw new IllegalArgumentException();

	    if (s.charAt(0) == '0') reMenuItem.setSelected(false);
	    if (s.charAt(1) == '1') imMenuItem.setSelected(true);
	}

	public JPopupMenu getJPopupMenu(ActionListener al) {

	    JPopupMenu p = PresentationFactory.getJPopupMenu(this.p, al);
	    p.addSeparator();
	    p.add(reMenuItem); reMenuItem.addActionListener(al);
	    p.add(imMenuItem); imMenuItem.addActionListener(al);
	    return p;
	}

	public int getRed(int pixel) {return pixel;}
	public int getBlue(int pixel) {return pixel;}
	public int getGreen(int pixel) {return pixel;}
	public int getAlpha(int pixel) {return 255;}

	public Image getImage() {

	    Layer l = reMenuItem.isSelected() ? p.getLayer(0) : p.getLayer(1);

	    int width = l.getWidth();	    int height = l.getHeight();
	    double min = l.getMinimum();    double max = l.getMaximum();	    
	    final int cpp = 255;	    double m = cpp / (max - min);
	    int[] id = new int[width * height];

	    for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++)
		    id[y * width + x] = (int) ((l.data[x][y] - min) * m);

	    MemoryImageSource mis = new MemoryImageSource(width, height, this, id, 0, width);
	    return Toolkit.getDefaultToolkit().createImage(mis);
	}

	public PresentationType getPresentationType() {return pt;}

	public Presentation getCopy(Picture p) {

	    if (!isCompatible(p))
		return PresentationFactory.createDefaultPresentation(p);
	    
	    MyPresentation mp = new MyPresentation(p);
	    mp.reMenuItem.setSelected(reMenuItem.isSelected());
	    mp.imMenuItem.setSelected(imMenuItem.isSelected());
	    return mp;
	}

	public String toString() {

	    return getName() + "$" + 
		(reMenuItem.isSelected() ? "1" : "0") +
		(imMenuItem.isSelected() ? "1" : "0");
	}
    }

    public ComplexPresentation() {}

    public boolean isCompatible(Picture mlp) {return mlp.getNumberOfLayers() == 2;}

    public String getName() {return "Komplexe Darstellung";}

    public Presentation createPresentation(Picture p) throws PictureException {

	if (!isCompatible(p))
	    throw new PictureException("Das Bild enthlt keine komplexen Daten");
	
	return new MyPresentation(p);
    }

    public Presentation createPresentation(Picture p, String s)
	throws PictureException, IllegalArgumentException {

	if (!isCompatible(p))
	    throw new PictureException("Das Bild enthlt keine komplexen Daten");

	return new MyPresentation(p, s);
    }
}

