/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;

public class DefaultPresentation implements PresentationType {

    private PresentationType pt = this;
    
    private final class MyPresentation extends ColorModel implements Presentation {

	private final Picture p;
	private Image img = null;

	public MyPresentation(Picture p) {super(31); this.p = p;}

	public JPopupMenu getJPopupMenu(ActionListener al) {

	    return PresentationFactory.getJPopupMenu(this.p, al);
	}

	public int getRed(int pixel) {return pixel;}
	public int getBlue(int pixel) {return pixel;}
	public int getGreen(int pixel) {return pixel;}
	public int getAlpha(int pixel) {return 255;}

	public Image getImage() {

	    if (img != null) return img;

	    Layer l = p.getLayer(0);
	    int width = l.getWidth();	    int height = l.getHeight();
	    double min = l.getMinimum();    double max = l.getMaximum();	    
	    final int cpp = 255;	    double m = cpp / (max - min);
	    int[] id = new int[width * height];

	    for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++)
		    id[y * width + x] = (int) ((l.data[x][y] - min) * m);

	    MemoryImageSource mis = new MemoryImageSource(width, height, this, id, 0, width);
	    img = Toolkit.getDefaultToolkit().createImage(mis);

	    return img;
	}

	public PresentationType getPresentationType() {return pt;}

	public Presentation getCopy(Picture p) {return new MyPresentation(p);}

	public String toString() {

	    return getName() + "$";
	}
    }

    public DefaultPresentation() {}

    public boolean isCompatible(Picture mlp) {return true;}

    public String getName() {return "Standard-Darstellung";}

    public Presentation createPresentation(Picture p) {return new MyPresentation(p);}

    public Presentation createPresentation(Picture p, String s) {return new MyPresentation(p);}
}
