/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

import javax.swing.JFrame;
import java.io.Serializable;
import picture.gui.*;

/**
 * Eine Transformation bildet ein Layer auf ein anderes Layer ab. Beispiel:
 * Die Bestimmung der Fouriertransformierten des Pictures <tt>p</tt> koennte mit
 * Hilfe einer Unterklasse <tt>FFT</tt> etwa wie folgt realisiert werden:
 * <pre>
 *  Picture fftVonP = (new FFT()).of(p);
 * </pre>
 * Etwaige Parameter koennten z.B. im Konstruktor uebergeben werden. 
 * Haengt eine Transformation von zwei (oder mehr) Bildern ab, 
 * so ist eine Realisierung in folgender Gestalt denkbar:
 * <pre>
 *  Picture a, b;
 *  Picture aSternB = (new Falter(a)).of(b);
 * </pre>
 */
public interface Transformation extends Serializable {

    /**
     * Liefert das Layer, das entsteht, wenn man diese Transformation auf <tt>p</tt>
     * anwendet.
     * @param p: Argument
     * @return <i>T</i>(<tt>p</tt>), wobei <i>T</i> = <tt>this</tt>
     */
    Picture of(Picture p) throws PictureException;

    /**
     * Liefert einen Optionendialog zurck (<tt>null</tt>, falls es keinen gibt).
     *
     * @param owner Parentframe
     * @param pic zu berbeitendes Bild
     * @return Optionendialog
     */
    DialogPane filterOptionPane(JFrame owner, Picture pic) throws PictureException;
}

