/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filetype;

import picture.*;

public interface JippPictureProperty extends PictureProperty {

    /**
     * Gibt an, ob zunaechst eine Wavelet-Zerlegung des Bildes
     * erfolgen soll.
     */
    public boolean waveletCompression();

    /**
     * Im Fall <tt>waveletCompression() == true</tt> gibt diese Methode an,
     * ob bei der Zerlegung des Bildes die Details hochskaliert werden sollen.
     * Das Skalieren der Details erhoeht Bildqualitaet und Dateigroesse
     * gleichermassen.
     */
    public boolean waveletDetailsScaling();

    /**
     * Im Fall <tt>waveletCompression() == true</tt> gibt diese Methode an,
     * wieviele Iterationen durchgefuehrt werden sollen.
     */
    public int getTotalNumberOfIterationsInWaveletAnalysis();

    /**
     * Im Fall <tt>waveletCompression() == true</tt> gibt diese Methode an,
     * wieviele Iterationen durchgefuehrt werden sollen ohne Beachtung der
     * Details. 
     */
    // Laut Tichy sollte ein Bezeichner nicht mehr als 60 Zeichen haben.
    // der folgende hat 52 ;-)
    public int getNumberOfIterationsWithoutDetailsInWaveletAnalysis();

    /**
     * Die erzeugte Enumeration enthalte Strings der Klassennamen von
     * FilterOutputStreams ohne das abschlieende OutputStream in der
     * Reihenfolge, wie sie vom Speicheralgorithmus abgearbeitet
     * werden sollen. */
    public java.util.Enumeration getCodingStreams();

    /**
     * Gibt den Kommentar, der im Header dieses Bildes gespeichert werden soll,
     * an.
     */
    public String[] getComment();

    /**
     * Gibt an, ob eine Checksumme der Bilddaten mit abgelegt werden
     * soll, wenn ja, welche, sonst null. 
     */
    public String getChecksum();

    /**
     * Gibt an, ob der Header komprimiert werden soll. 
     */
    public boolean compressHeader();

    /**
     * Gibt an, ob digitisiert gespeichert werden soll.
     */
    public boolean digitize();

    /**
     * Gibt den maximalen Grauwert des Bildes an.
     * Der minimale Grauwert des Bildes ist stets 0, somit wird
     * ueber diese Methode die Anzahl der Graustufen und damit
     * insbesondere die Groesse des Bildes festgelegt.
     *
     * Wird nur beachtet, falls digitize() == true!
     */
    public long getNumberOfGrayScales(); 
}
