/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filter;

import javax.swing.JFrame;
import java.util.*;

import picture.gui.*;
import picture.*;

/**
 * Diese Klasse implementiert Morphfilter.
 *
 * Das sind Filter, bei denen der neue Wert eines Pixels durch die
 * umgebenden bestimmt wird. Ergebis ist ein reelles Bild, das aus der
 * ausgewhlten Prsentation gewonnen wurde. 
 */
public class MorphFilter extends DefaultTransformation {

    public static final int MEDIAN = 1;
    public static final int MIN = 2;
    public static final int MAX = 3;

    private int type;

    protected int r = 1;

    public MorphFilter() {this(MEDIAN);}

    public MorphFilter(int type) {this.type = type;}

    public int getRadius() {return r;}

    public void setRadius(int r) {this.r = r;}

    public int getType() {return type;}

    public void setType(int t) {type = t;}

    private final double getMorphFilteredPixel(Layer p, int x, int y) {

	int r = this.r;	// Threads ...
	double[] s = new double[(2*r+1)*(2*r+1)];
	int w = p.getWidth(), h = p.getHeight();

	int pos = 0;
	
	for(int i = x-r; i <= x+r; i++)
	    for(int j = y-r; j <= y+r; j++)
		if(i >= 0 && i < w && j >= 0 && j < h)
		    s[pos++] = p.getValue(i, j);

	if (pos > 0) {
	    Arrays.sort(s, 0, pos);
	    int t;
	    switch(type) {
	    case MIN: t = 0; break;
	    case MAX: t = pos - 1; break;
	    default:  t = pos / 2; break;
	    }
	    return s[t];
	} 
	
	return 0;
    }

    public int getNumberOfProgressSteps(Picture p) {

	return p.getWidth() * p.getNumberOfLayers();
    }

    public Layer of(ProgressHandler ph, Layer p) throws PictureException {

	int w = p.getWidth(), h = p.getHeight();

	double[][] data = new double[w][h];

	for(int x = 0; x < w; x++) {
	    for(int y = 0; y < h; y++)
		data[x][y] = getMorphFilteredPixel(p, x, y);

	    if (ph != null) ph.step();
	}

	return createNewInstance(p, data);
    }

    /**
     * Liefert einen Optionendialog zurck.
     *
     * @param owner Parentframe
     * @param pic zu berbeitendes Bild
     * @return OptionenDialog
     */
    public DialogPane filterOptionPane(JFrame owner, Picture pic) 
	throws PictureException {
	return new MorphPane(owner, pic, this);
    }
}
