/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filter;
import picture.*;

/**
 * Diese Klasse repraesentiert eine Rotation des Bildes um ein ganzahliges Vielfaches
 * von neunzig Grad.
 */
public class Rotation extends DefaultTransformation {

    /* n = 0: Identitaet
     * n = 1: 90 Grad nach links
     * n = 2: 180 Grad
     * n = 3: 90 Grad nach rechts
     */
    private int n = 0;

    /**
     * Liefert eine Drehung um 0 * 90 Grad, also die Identitaet.
     */
    public Rotation() {}

    /**
     * Erzeugt eine Drehung um n * 90 Grad entgegen dem Uhrzeigersinn.
     */
    public Rotation(int n) {

	while (n < 0) n += 4;
	this.n = n % 4;
    }

    public int getNumberOfProgressSteps(Picture p) {return 0;}

    public Layer of(ProgressHandler ph, Layer p) {

	double [][] data = p.data;
	int w = p.getWidth(); 
	int h = p.getHeight();
	
	switch (n) {
	case 1: // 90 Grad nach links
	    data = new double[h][w];
	    for (int i = 0; i < h; i++)
		for (int j = 0; j < w; j++)
		    data[i][j] = p.data[w - 1 - j][i];
	    break;
	case 2: // 180 Grad
	    data = new double[w][h];
	    for (int i = 0; i < w; i++)
		for (int j = 0; j < h; j++)
		    data[i][j] = p.data[w - 1 - i][h - 1 - j];
	    break;
	case 3: // 90 Grad nach rechts
	    data = new double[h][w];
	    for (int i = 0; i < h; i++)
		for (int j = 0; j < w; j++)
		    data[i][j] = p.data[j][h - 1 - i];
	    break;
	}
	return createNewInstance(p, data);
    }
}
