/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import picture.*;
import picture.gui.*;
import picture.filter.*;
import picture.math.Term;
import picture.filetype.*;

/**
 * Diese Klasse erlaubt die Einstellungen zur Hoch-, Tief- und Bandpassfilterung von Bildern
 */
public class FilterPane extends DialogPane {

    private static int filterdateinummer = 0; // zur Dateinamengenerierung

    private static final int HOCHPASS = PassFilter.HIGHPASSFILTER; 
    private static final int TIEFPASS = PassFilter.LOWPASSFILTER;
    private static final int BANDPASS = PassFilter.BANDPASSFILTER;
    private static final int DATEI = 10;
    private static final int FREI = 11;
    private static final int FORMEL = FREI;

    /**
     * Mit dieser Klasse wird das Ring-Bildchen dargestellt.
     * Sie beinhaltet gleichzeitig die Implementierungen der noetigen Eventlistener
     */
    private class FilterPreview extends JComponent implements ChangeListener, ActionListener {
	
	public Color kreisfarbe = Color.red;

	public FilterPreview() {
	
	    updater.add(filter);
	    setDoubleBuffered(true);
	}

	public Dimension getPreferredSize() {return new Dimension(64, 84);}

	public Dimension getMinimumSize() {return getPreferredSize();}

	public void stateChanged(ChangeEvent e) {

	    int f = filterart;
	    if (bandpass.isSelected()) {
		f = BANDPASS;
		cl.show(this0, "generisch"); 
		minRadius.setEnabled(true); maxRadius.setEnabled(true);
	    } else if (hochpass.isSelected()) {
		f = HOCHPASS;
		cl.show(this0, "generisch");
		minRadius.setEnabled(true); maxRadius.setEnabled(false);
	    } else if (tiefpass.isSelected()) {
		f = TIEFPASS;
		cl.show(this0, "generisch");
		minRadius.setEnabled(false); maxRadius.setEnabled(true);
	    } else if (dateiFilter.isSelected()) {
		f = DATEI;
		cl.show(this0, "datei");	
	    } else if (freierFilter.isSelected()) {
		f = FREI;
		cl.show(this0, "frei");
	    }

	    boolean anders = false;
	    if (f != filterart) {
		filterart = f;
		anders = true;
		minRadius.repaint();
		maxRadius.repaint();
	    }

	    if (datei != dateiFilter.isSelected()) {
		datei = !datei;
		anders = true;

		if (datei && filterdatei == null) 
		    filterdateiwaehlen();
		if (filterdatei == null)
		    dateiFilter.setSelected(false);
	    }

	    if (graduell != graduellerFilter.isSelected()) {
		graduell = !graduell;
		anders = true;
	    }

	    double min = kleinerRadius; 
	    double max = grosserRadius;

	    if (e.getSource() == minRadius) {
		anders = true;
		setzeKleinenRadius(minRadius.getValue() / 100.0);
		maxRadius.setValue((int) (grosserRadius * 100));
	    } else if (e.getSource() == maxRadius) {
		anders = true;
		setzeGrossenRadius(maxRadius.getValue() / 100.0);
		minRadius.setValue((int) (kleinerRadius * 100));
	    }

	    if (anders || min != kleinerRadius || max != grosserRadius) {
		update();
		repaint();
	    }
	}

	private void update() {

	    switch (filterart) {
	    case HOCHPASS:
	    case TIEFPASS:
	    case BANDPASS:
		try {
		    filter.setBounds(kleinerRadius, grosserRadius);
		    filter.setMode(filterart);
		    filter.setGraduell(graduell);
		} catch (IllegalArgumentException e) {}
		break;
	    case DATEI:
		filter.setPicture(filterPreviewPicture); break;
	    case FORMEL:
		try {
		    filter.setFunction(new Term(formel.getText()));
		} catch (IllegalArgumentException e) {
		    GUIHelp.error(e, "Syntaxfehler");
		} catch (ArithmeticException e) {
		    GUIHelp.error(e);
		}
		break;
	    }

	    updater.add(filter); 
	    repaint();
	}

	public void actionPerformed(ActionEvent e) {

	    String k = e.getActionCommand();
	    if (k.equals("speichern")) {

		Picture f = null;
		try {
		    f = (Picture)
			((PassFilter)getTransformation()).createFilterPicture(64, 64);
		} catch (Exception ex) {
		    GUIHelp.error(ex);
		    return;
		}
		 
		if (f == null) {
		    GUIHelp.error(this1, "Die aktuelle Konfiguration fhrt nicht\n" +
				"zu einem sinnvollen Filter.");
		    return;
		}
		
		String name = (new File(".")).getPath();
		name = name.substring(0, name.length() - 1);
		name += "filter" + (++filterdateinummer) + ".pgm";
		    
		try {
		    f.setFileName(name);
		    GUIHelp.saveTo(this1, f, new File(name));
		} catch (PictureException ex) {
		    GUIHelp.error(ex, "Fehler beim Schreiben der Datei.");
		}
		
	    } else if (k.equals("durchsuchen")) 
		filterdateiwaehlen();
	    else if (e.getSource() == filterdateiname) 
		try {

		    File f = new File(filterdateiname.getText());
		    Picture p = PictureTypeFactory.loadPicture(f);

		    filterdatei = f;
		    filterPicture = p;
		    filterPreviewPicture = p.getPreview();
		} catch (IOException ex) {
		    GUIHelp.error(ex, "Die angegebene Datei ist ungltig\n" +
				"oder existiert nicht.");
		    filterdateiname.setText(filterdatei.getPath());
		} catch (PictureException ex) {
		    GUIHelp.error(ex, "Die angegebene Datei ist ungltig\n" +
				"oder existiert nicht.");
		    filterdateiname.setText(filterdatei.getPath());
		}
	    
	    update();
	    repaint();
	}

	private void filterdateiwaehlen() {

	    boolean ging = true;

	    String alteDatei = filterdateiname.getText();
	    if (alteDatei.length() == 0) 
		alteDatei = ".";
	    
	    do {
		ging = true;
		File f = GUIHelp.openFile(this1, alteDatei);
		if (f != null) 
		    filterdatei = f;

		if (filterdatei == null)
		    return;

		try {
		    filterPicture = PictureTypeFactory.loadPicture(filterdatei);
		    filterPreviewPicture = filterPicture.getPreview();
		} catch (IOException ex) {
		    GUIHelp.error(ex, "Fehler beim Laden der Datei\n" + filterdatei.getName());
		    ging = false;
		} catch (PictureException ex) {
		    GUIHelp.error(ex, "Fehler beim Laden der Datei\n" + filterdatei.getName());
		    ging = false;
		}
	    } while (!ging);

	    filterdateiname.setText(filterdatei.getPath());
	}

	public void setzeKleinenRadius(double d) {

	    if (d < 0) d = 0;
	    if (d > 1) d = 1;
	    
	    kleinerRadius = d;
	    if (grosserRadius < kleinerRadius)
		grosserRadius = kleinerRadius;

	    repaint();
	}

	public void setzeGrossenRadius(double d) {

	    if (d < 0) d = 0;
	    if (d > 1) d = 1;

	    grosserRadius = d;
	    if (kleinerRadius > grosserRadius)
		kleinerRadius = grosserRadius;

	    repaint();
	}

	public void paint(Graphics g) {

	    Dimension d = getSize(); d.height -= 20;

	    int rx = d.width / 2; int ry = d.height / 2; int rm;
	    
	    int links = rx - fftPreview.getWidth() / 2;
	    int rechts = rx + fftPreview.getWidth() / 2;
	    int oben = ry - fftPreview.getHeight() / 2;
	    int unten = ry + fftPreview.getHeight() / 2;

	    // Zeichne jetzt das Bildchen mit den Ringen

	    try {
		if (filterart == DATEI) {
		    g.drawImage(filterPreviewPicture.getImage(), links, oben, this);
		} else if (filterart == FORMEL) {

		    Term t = new Term(formel.getText());
		    String [] x = new String[]{"x"};
		    Complex [] z = new Complex[1];
		    double skalierung = Math.sqrt(2) / (rechts - links);
		    int mx = (links + rechts) / 2;
		    int my = (oben + unten) / 2;
		    for (int i = (int) (1 / skalierung); i > 0; i--) {

			z[0] = new Complex(skalierung * i, 0);
			z[0] = t.einsetzen(x, z);
			if (z[0] == null)
			    throw new Exception();
			
			int red = (int) (255 * z[0].getRealPart());
			if (red > 255) red = 255; 
			if (red < 0) red = 0;
			g.setColor(new Color(red, 0, 0));
			g.fillOval(mx - i, my - i, 2 * i, 2 * i);
		    }
		    g.setColor(getBackground());
		    g.fillRect(0, 0, d.width, oben);
		    g.fillRect(0, unten, d.width, d.height);
		    g.fillRect(0, 0, links, d.height);
		    g.fillRect(rechts, 0, d.width, d.height);		    

		} else {
		    
		    g.drawImage(fftPreviewImage, links, oben, this);
		    if (filterart != TIEFPASS) {
			// Ein ausgefuellter Kreis um den Mittelpunkt wird auf schwarz
			// gesetzt
			g.setColor(Color.black);
			rm = berechneRadius(kleinerRadius);
			g.fillOval(rx - rm, ry - rm, 2 * rm, 2 * rm);
			g.setColor(kreisfarbe); 
			g.drawOval(rx - rm, ry - rm, 2 * rm, 2 * rm);
		    } 
		    
		    if (filterart != HOCHPASS) {
			// ein Kreis um den Mittelpunkt bleibt stehen
			g.setColor(Color.black);
			rm = berechneRadius(grosserRadius);
			int[] x = new int[6 + rm]; int[] y = new int[6 + rm];
			x[0] = rx; y[0] = unten;
			x[1] = links; y[1] = unten;
			x[2] = links; y[2] = oben;
			x[3] = rechts; y[3] = oben;
			x[4] = rechts; y[4] = unten;
			x[5] = rx; y[5] = unten;
			for (int i = 6; i < 6 + rm; i++) {
			    x[i] = rx - (int) (rm * Math.sin((i - 6) * 2 * Math.PI / (rm - 1)));
			    y[i] = ry + (int) (rm * Math.cos((i - 6) * 2 * Math.PI / (rm - 1)));
			}
			g.fillPolygon(x, y, 6 + rm);
			g.setColor(kreisfarbe); 
			g.drawOval(rx - rm, ry - rm, 2 * rm, 2 * rm);		    
		    }
		}
	    } catch (Exception e) {

		// Schwarzes Kreuz auf weissem Grund.
		g.setColor(Color.white);
		g.fillRect(links, oben, rechts - links, unten - oben);
		g.setColor(Color.black);
		g.drawLine(links, oben, rechts, unten);
		g.drawLine(links, unten, rechts, oben);
	    }

	    // Zeichne nun den Graduellen bzw. diskreten Verlauf in den Bereich
	    // (0, d.height + 4) x (d.width, d.height + 20)

	    if (d.width < 2 || filterart == DATEI)
		return;     

	    int yOffset = d.height + 4;
	    g.setColor(Color.black); g.fillRect(0, yOffset, d.width, 16);
	    g.setColor(Color.green);
	    
	    int nul = yOffset + 14;
	    int eins = yOffset + 2;
	    int y0; int k = eins - nul; double l = 2.0 / d.width;
	    double wert; int yAlt;
	    double untergrenze = d.width * (1 - kleinerRadius) / 2;
	    double obergrenze = d.width * (1 - grosserRadius) / 2;

	    switch (filterart) {
	    case BANDPASS: 
		wert = graduell ? FilterPicture.bandpass(1, kleinerRadius, grosserRadius) : 0;
		yAlt = nul + (int)(k * wert);
		for (int x = 1; x < d.width / 2 + 1; x++) {    
		    wert = graduell 
			? FilterPicture.bandpass(1 - l * x, kleinerRadius, grosserRadius)
			: ((x >= untergrenze || x <= obergrenze) ? 0 : 1);
		    y0 = nul + (int)(k * wert);
		    g.drawLine(x - 1, yAlt, x, y0);
		    g.drawLine(d.width - x - 1, yAlt, d.width - x - 2, y0);
		    yAlt = y0;
		}
		break;
	    case TIEFPASS: 
		wert = graduell ? FilterPicture.lowpass(1, grosserRadius) : 0;
		yAlt = nul + (int) (k * wert);
		for (int x = 1; x < d.width / 2 + 1; x++) {
		    wert = graduell
			? FilterPicture.lowpass(1 - l * x, grosserRadius) 
			: ((x <= obergrenze) ? 0 : 1);
		    y0 = nul + (int) (k * wert);
		    g.drawLine(x - 1, yAlt, x, y0);
		    g.drawLine(d.width - x - 1, yAlt, d.width - x - 2, y0);
		    yAlt = y0;
		}
		break;
	    case HOCHPASS: 
		wert = graduell ? FilterPicture.highpass(1, kleinerRadius) : 1;
		yAlt = nul + (int) (k * wert);
		for (int x = 1; x < d.width / 2 + 1; x++) {
		    wert = graduell 
			? FilterPicture.highpass(1 - l * x, kleinerRadius) 
			: ((x >= untergrenze) ? 0 : 1);
		    y0 = nul + (int) (k * wert);
		    g.drawLine(x - 1, yAlt, x, y0);
		    g.drawLine(d.width - x - 1, yAlt, d.width - x - 2, y0);
		    yAlt = y0;
		}
		break;
	    case FORMEL:
		try {
		    Term t = new Term(formel.getText());
		    String [] x = new String[]{"x"};
		    Complex [] z = new Complex[1];

		    double skalierung = 2.0 / d.width;
		    z[0] = new Complex(1, 0);
		    z[0] = t.einsetzen(x, z);
		    if (z[0] == null)
			break;
		    yAlt = nul + (int) (k * z[0].getRealPart());
		    for (int i = 0; i < d.width / 2; i++) {
			z[0] = new Complex(1 - skalierung * i, 0);
			z[0] = t.einsetzen(x, z);
			y0 = nul + (int) (k * z[0].getRealPart());
			if (y0 > nul) y0 = nul;
			if (y0 < eins) y0 = eins;
			g.drawLine(i - 1, yAlt, i, y0); 
			g.drawLine(d.width - i - 1, yAlt, d.width - i - 2, y0);
			yAlt = y0;
		    }
		} catch (Exception ign) {} // Division durch Null usw.
		break;
	    }
	}

	private int berechneRadius(double r) {

	    Dimension d = getSize();

	    int dx = (int) (r * d.width / 2);
	    int dy = (int) (r * d.height / 2);
	    
	    int rm = dx; 
	    if (dy < dx) rm = dy;

	    return rm;
	}
    }

    public int filterart = BANDPASS;
    public double kleinerRadius = 0.33;
    public double grosserRadius = 0.67;
    
    private boolean graduell = false;
    private boolean datei = false;

    private Picture pic;

    private PassFilter filter;
    
    // enthaelt fft ohne Mittelpunktsverschiebung (zum rechnen)
    private Picture fftPreview; 
    // fft mit Mittelpktversch. (zum Anzeigen)
    private Image fftPreviewImage;

    private File filterdatei;
    private Picture filterPicture;
    private Picture filterPreviewPicture;

    private JRadioButton hochpass = new JRadioButton("Hochpassfilter");
    private JRadioButton tiefpass = new JRadioButton("Tiefpassfilter");
    private JRadioButton bandpass = new JRadioButton("Bandpassfilter");
    private JRadioButton dateiFilter = new JRadioButton("Filterdatei");
    private JRadioButton freierFilter = new JRadioButton("frei");

    private JSlider minRadius = new JSlider(JSlider.HORIZONTAL, 0, 100, 33);
    private JSlider maxRadius = new JSlider(JSlider.HORIZONTAL, 0, 100, 66);
    private JCheckBox graduellerFilter = new JCheckBox("graduell");

    private JTextField formel = new JTextField("1");

    private JButton durchsuchen = new JButton("durchsuchen");
    private JTextField filterdateiname = new JTextField("", 20);

    CardLayout cl = new CardLayout();

    private FilterPreview ringe;

    private Container this0; 
    private Container this1;

    /**
     * Erzeugt ein neues FilterPane
     */
    public FilterPane(Frame owner, Picture pic) throws PictureException {
	this(owner, pic, new PassFilter(0.33, 0.66));
    }

    /**
     * Erzeugt ein neues FilterPane
     */
    public FilterPane(Frame owner, Picture pic, PassFilter pf) 
	throws PictureException {

	super(owner, "Filtereinstellungen", pic);

	/*
	 * Filter retten
	 */

	filter = pf;
	Term oldTerm = pf.getFunction();
	double [] oldBounds = pf.getBounds();
	int oldMode = pf.getMode();
	Picture oldPic = (Picture)/*hoffentlich*/pf.getPicture();
	boolean oldGraduell = pf.isGraduell();

	/*
	 * Vorarbeit
	 */

	fftPreview = (new FFT()).of(pic.getPreview());
	fftPreviewImage = fftPreview.getShiftedInstance(32, 32).getImage();
	ringe = new FilterPreview();

	/*
	 *   Erzeugung der Komponenten
	 */

	Container rahmen = getContentPane();

	JPanel filterwahl = new JPanel(new GridLayout(0, 1));
	JPanel ringPanel = new JPanel(new BorderLayout(10, 10));
	JPanel werkzeuge = new JPanel(cl);
	this0 = werkzeuge;
	this1 = this;

	JPanel generischPanel = new JPanel(new GridLayout(0, 1, 10, 10));

	JPanel dateiPanel = new JPanel(new BorderLayout(10, 10));
	JPanel dateiWahl = new JPanel(new BorderLayout());
	JPanel dateiWahl1 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
	JPanel dateiText = new JPanel(new GridLayout(0, 1));

	JPanel formelPanel = new JPanel(new BorderLayout(10, 10));
	JPanel formelText = new JPanel(new GridLayout(0, 1));

	formel.setFont(new Font("Monospaced", Font.PLAIN, 14));

	/*
	 *   Layout
	 */

	filterwahl.add(hochpass); 
	filterwahl.add(tiefpass); 
	filterwahl.add(bandpass); 
	filterwahl.add(freierFilter);
	filterwahl.add(dateiFilter);
	
	werkzeuge.add("generisch", generischPanel);
	werkzeuge.add("frei", formelPanel);
	werkzeuge.add("datei", dateiPanel);

	generischPanel.add(new JLabel("Innenring"));
	generischPanel.add(minRadius);
	generischPanel.add(new JLabel("Auenring"));
	generischPanel.add(maxRadius);
	generischPanel.add(graduellerFilter);

	dateiText.add(new JLabel("Whlen Sie die Datei aus,"));
	dateiText.add(new JLabel("deren Inhalt als Filterfunktion"));
	dateiText.add(new JLabel("verwendet werden soll."));

	dateiPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
	dateiPanel.add("Center", dateiText);
	dateiPanel.add("South", dateiWahl);
	dateiWahl.add("North", filterdateiname);
	dateiWahl.add("South", dateiWahl1);
	dateiWahl1.add(durchsuchen);

	formelText.add(new JLabel("Geben Sie einen von x abhngigen Term"));
	formelText.add(new JLabel("ein. Durch diesen Term wird auf dem"));
	formelText.add(new JLabel("Intervall [0, 1] die verwendete"));
	formelText.add(new JLabel("Filterfunktion definiert."));
	formelText.add(new JPanel());

	formelPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
	formelPanel.add("Center", formelText);
	formelPanel.add("South", formel);

	ringPanel.add("Center", ringe);
	
 	rahmen.add("West", filterwahl);
	rahmen.add("Center", werkzeuge);
	rahmen.add("East", ringPanel); 

	/*
	 *  Verkabelung
	 */
	
	formel.addActionListener(ringe);
	filterdateiname.addActionListener(ringe);

	minRadius.addChangeListener(ringe);
	maxRadius.addChangeListener(ringe);

	hochpass.addChangeListener(ringe);
	tiefpass.addChangeListener(ringe);
	bandpass.addChangeListener(ringe);
	freierFilter.addChangeListener(ringe);
	dateiFilter.addChangeListener(ringe);

	graduellerFilter.addChangeListener(ringe);

	durchsuchen.addActionListener(ringe);

	ButtonGroup bg = new ButtonGroup();
	bg.add(hochpass); bg.add(tiefpass); 
	bg.add(bandpass); bg.add(dateiFilter);
	bg.add(freierFilter);

	/*
	 * Initialisierung
	 */

	graduell = oldGraduell;
	graduellerFilter.setSelected(oldGraduell);

	if (oldPic != null) {
	    filterdatei = new File("."); filterdateiname.setText("?");
	    filterPicture = oldPic; filterPreviewPicture = oldPic.getPreview();
	}

	switch (oldMode) {
	case PassFilter.HIGHPASSFILTER: hochpass.setSelected(true); break;
	case PassFilter.LOWPASSFILTER: tiefpass.setSelected(true); break;
	case PassFilter.BANDPASSFILTER: bandpass.setSelected(true); break;
	case PassFilter.FILE: 
	    datei = true; dateiFilter.setSelected(true); cl.show(this0, "datei"); 
	    break;
	case PassFilter.TERM: freierFilter.setSelected(true); cl.show(this0, "frei"); break;
	}

	kleinerRadius = oldBounds[0]; minRadius.setValue((int) (kleinerRadius * 100));
	grosserRadius = oldBounds[1]; maxRadius.setValue((int) (grosserRadius * 100));

	if (oldTerm != null) {
	    formel.setText(oldTerm.toString());
	}

	ringe.update();
	prepareWindow();
	ringe.repaint();
    }

    public Transformation getTransformation() {return filter;}

    /**
     * Gibt das Pictureobjekt zurueck, das entsteht, wenn man die 
     * Filterung anhand der Benutzereinstellungen durchfuehrt.
     * @exception PictureException, falls FFT versagt.
     */
    public Picture getResultingPicture() throws PictureException {

	Picture pic = getSource();
	ProgressHandler ph = pic.getProgressHandler();
	if (ph != null) 
	    ph.setBounds(0, 4 * pic.getWidth());

	PassFilter pf = (PassFilter) getTransformation();

	if (pf == null)
	    throw new PictureException("Fehler bei der Erzeugung des Filters");

	return pf.of(pic);
    }
}
