/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

import picture.*;

/**
 * Anzeigekomponente fr Layer-Objekte.
 */
public class JPicture extends JComponent {

    public PictureModel img;
    private boolean modified = false;
    private double zoomFactor = 1; 
    private Image image;

    private int shiftX = 0;
    private int shiftY = 0;

    public JPicture() {}

    public JPicture(PictureModel img) {

	setPictureModel(img);
    }

    public void updateImage() {image = getImage();}

    public Image getImage() {

	Image img = this.img.getImage();
	if (img != null)
	    img = img.getScaledInstance((int) (zoomFactor * this.img.getWidth()),
					(int) (zoomFactor * this.img.getHeight()),
					Image.SCALE_DEFAULT);
	return img;
    }

    public void setPictureModel(PictureModel pm) {

	this.img = pm;
	image = getImage();
    }

    public PictureModel getPictureModel() {return img;}

    public void setModified(boolean b) {modified = b;}

    public boolean isModified() {return modified;}

    public void paintComponent(Graphics g) {

	if (image == null)
	    image = getImage();
	if (image == null) 
	    return;

	Dimension d = getSize();
	int w = (int) (img.getWidth() * zoomFactor);
	int h = (int) (img.getHeight() * zoomFactor);
	
	for (int sx = -1; sx < 1; sx++)
	    for (int sy = -1; sy < 1; sy++) 
		g.drawImage(image, 
			    sx * w + (d.width - w) / 2 + shiftX, 
			    sy * h + (d.height - h) / 2 + shiftY, 
			    this);	

	g.setColor(getBackground());
	g.fillRect(0, 0, d.width, (d.height - h) / 2);
	g.fillRect(0, 0, (d.width - w) / 2, d.height);
	g.fillRect(0, (d.height + h) / 2, d.width, d.height);
	g.fillRect((d.width + w) / 2, 0, d.width, d.height);
    }

    public String getToolTipText(MouseEvent e) {

	Dimension d = getSize();
	double w = img.getWidth() * zoomFactor;
	double h = img.getHeight() * zoomFactor;
	int x = (int) ((e.getX() - (d.width - w) / 2) / zoomFactor); 
	int y = (int) ((e.getY() - (d.height - h) / 2) / zoomFactor);

	String s = "(" + x + ", " + y + ")";
	String info = null;

	if (x >= 0 && x < img.getWidth() && y >= 0 && y < img.getHeight())
	    info = img.getInfo(x, y);

	if (info != null)
	    s = s + " " + info;

	return s;
    }

    public void setShift(int x, int y) {

	shiftX = x; shiftY = y;
	while (shiftX < 0)
	    shiftX += img.getWidth();
	while (shiftY < 0)
	    shiftY += img.getHeight();

	shiftX = shiftX % img.getWidth();
	shiftY = shiftY % img.getHeight();

	repaint();
    }

    public Dimension getPreferredSize() {

	return new Dimension((int) (zoomFactor * img.getWidth()), 
			     (int) (zoomFactor * img.getHeight()));
    }

    public void setZoomFactor(double f) {

	zoomFactor = Math.abs(f);
	setSize(getPreferredSize());
	image = getImage();
	repaint();
    }

    public double getZoomFactor() {return zoomFactor;}
}
