/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import picture.*;
import picture.filter.*;

/**
 * Diese Klasse erlaubt die Einstellungen zur Morphfilterung.
 */
public class MorphPane extends DialogPane {

    private JComboBox filterArt = new JComboBox();
    private MorphFilter median;
    private JSlider groesse;
    private EventListener ev = new EventListener();

    private class EventListener implements ChangeListener, ItemListener {

	public EventListener() {}

	public void itemStateChanged(ItemEvent e) {

	    int t = filterArt.getSelectedIndex() + 1;
	    if (median.getType() != t) {
		median.setType(t); 
		updater.add(median);
	    }
	}

	public void stateChanged(ChangeEvent e) {
	    
	    int r = groesse.getValue();
	    if (median.getRadius() != r) {
		median.setRadius(r);
		updater.add(median);
	    }
	}
    };

    /**
     * Erzeugt ein neues MorphPane.
     */
    public MorphPane(Frame owner, Picture pic) throws PictureException {
	this(owner, pic, new MorphFilter());
    }

    /**
     * Erzeugt ein neues MorphPane aus gegebenem MorphFilter.
     */
    public MorphPane(Frame owner, Picture pic, MorphFilter med) throws PictureException {

	super(owner, "Morphfilter", pic);
	median = med;
	
	/*
	 *  Erzeugung der Komponenten
	 */
	groesse = new JSlider(JSlider.HORIZONTAL, 0, 20, median.getRadius());

	Container mitte = getContentPane();
	mitte.add("West", filterArt);
	mitte.add("South", groesse);

	/*
	 *  Verkabelung
	 */
	filterArt.addItem("Median");
	filterArt.addItem("Minimum");
	filterArt.addItem("Maximum");
	filterArt.addItemListener(ev);

        groesse.setMajorTickSpacing(2);
        groesse.setMinorTickSpacing(1);
	groesse.setSnapToTicks(true);
        groesse.setPaintTicks(true);
        groesse.setPaintLabels(true);
	groesse.addChangeListener(ev);

	prepareWindow();
	updater.add(median);
    }

    /** Gibt die Transformation aus den aktuellen Einstellungen zurueck */
    public Transformation getTransformation() {return median;}
}
