/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import picture.*;
import picture.filter.*;

/**
 * Anzeigen der Shiftvorschau.
 */
public class ShiftPane extends DialogPane {

    public class EventListener implements ActionListener, ChangeListener {

	public void stateChanged(ChangeEvent e) {

	    if (e.getSource() == stt) {
		boolean b = stt.isSelected();
		horizontal.setSnapToTicks(b);
		horizontal.setPaintTicks(b);
		vertikal.setSnapToTicks(b);
		vertikal.setPaintTicks(b);
	    } else if (e.getSource() == horizontal || e.getSource() == vertikal) {
		int w = prev.getWidth() * horizontal.getValue() / pic.getWidth();
		int h = prev.getHeight() * vertikal.getValue() / pic.getHeight();
		anzeige.setShift(prev.getWidth() / 2 + w, prev.getHeight() / 2 - h);
	    }
	}

	public void actionPerformed(ActionEvent e) {

	    if (e.getActionCommand().equals("OK")) {
		closingCode = OK; dispose();
	    } else if (e.getActionCommand().equals("Abbrechen")) {
		closingCode = ABBORD; dispose();
	    }		
	}
    }

    private Picture pic;
    private Picture prev;

    private JSlider horizontal;
    private JSlider vertikal; 

    private JPicture anzeige;

    private EventListener el = new EventListener();

    private JCheckBox stt = new JCheckBox("diskret");

    /**
     * Erzeugt ein neues ScalePane
     */
    public ShiftPane(Frame owner, Picture pic) throws Exception {

	super(owner, "Zyklisches Verschieben");

	if (pic == null)
	    throw new Exception("Das Bild ist ungltig.");

	int w = pic.getWidth();
	int h = pic.getHeight();
	horizontal = new JSlider(JSlider.HORIZONTAL, -w / 2, w / 2, -w / 2);
	vertikal = new JSlider(JSlider.VERTICAL, -h / 2, h / 2, -h / 2);
	
	this.pic = pic;
	this.prev = pic.getScaledInstance(horizontal.getPreferredSize().width,
					  vertikal.getPreferredSize().height);

	anzeige = new JPicture(prev);

	horizontal.addChangeListener(el);
	vertikal.addChangeListener(el);

	/*
	 *   Erzeugung der Komponenten
	 */

	Container c = getContentPane();

	JPanel mitte = new JPanel(new BorderLayout(0, 0));
	JPanel unten = new JPanel(new GridLayout(1, 0, 10, 10));

	JPanel skala0 = new JPanel(new FlowLayout(FlowLayout.LEFT));
	JPanel skala1 = new JPanel(new FlowLayout(FlowLayout.LEFT));

	JButton ok = new JButton("OK");
 	JButton abbruch = new JButton("Abbrechen");

	/*
	 *   Layout
	 */

	skala0.add(anzeige); skala0.add(vertikal); 
	skala1.add(horizontal);

	mitte.setBorder(new EmptyBorder(10, 10, 10, 10));
	mitte.add("North", skala0); 
	mitte.add("Center", skala1);
	mitte.add("South", stt);

	unten.setBorder(new EmptyBorder(10, 10, 10, 10));
	unten.add(ok);
	unten.add(abbruch);

	c.add("Center", mitte);	
	c.add("South", unten);

	pack();

	Toolkit tk = Toolkit.getDefaultToolkit();
	Dimension screen = tk.getScreenSize();
	Dimension size = getSize();
	setSize((int) (size.width * 1.1), size.height);
	size = getSize();
	setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);

	/*
	 *  Verkabelung
	 */

	ok.setActionCommand("OK"); ok.addActionListener(el);
	abbruch.setActionCommand("Abbrechen"); abbruch.addActionListener(el);	

	stt.setSelected(true);
	stt.addChangeListener(el);	

	horizontal.addChangeListener(el);
	horizontal.setMinorTickSpacing(w / 8);
	horizontal.setSnapToTicks(stt.isSelected());
	horizontal.setPaintTicks(stt.isSelected());

	vertikal.addChangeListener(el);
	vertikal.setMinorTickSpacing(h / 8);
	vertikal.setSnapToTicks(stt.isSelected());
	vertikal.setPaintTicks(stt.isSelected());

    }

    /**
     * Gibt das Pictureobjekt zurueck, das entsteht, wenn man die 
     * Filterung anhand der Benutzereinstellungen durchfuehrt.
     */
    public Picture getResultingPicture() {

	return 
	    pic.getShiftedInstance(pic.getWidth() / 2 + horizontal.getValue(), 
				   pic.getHeight() / 2 + vertikal.getValue());
    }
}

