/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import picture.*;
import picture.filter.*;
import picture.math.Term;

/**
 * Benutzerschnittstelle fuer die Interaktion mit kontinuierlichen Wavelets
 */
public class WaveletCenter extends DialogPane {

    private static final String [] x = new String[]{"x"};

    // #############################################################################
    // #############################################################################
    // #############################################################################

    private static final class Graph extends JComponent 
	implements MouseMotionListener, MouseListener{

	private final int breite = 128;

	private final double [] data = new double[breite];
	private final Complex [][] xComplex = new Complex[breite][1];
	private double max = 1;
	private Color farbe = Color.red;
	private boolean editierbar = true;

	public Graph() {
	    
	    setDoubleBuffered(true); 
	    setBorder(new EtchedBorder());
	    addMouseMotionListener(this);
	    addMouseListener(this);
	    for (int i = 0; i < breite; i++) 
		xComplex[i][0] = new Complex(i * 5.0 / (breite - 1) - 2.5, 0);
	}

	public Dimension getPreferredSize() {return new Dimension(breite, 3 * breite / 4);}
	public void setColor(Color f) {farbe = f;}
	public Color getColor() {return farbe;}
	public void setEditable(boolean b) {editierbar = b;}
	public boolean isEditable() {return editierbar;}

	public void setData(double[] d) {

	    max = 0;
	    for (int i = 0; i < d.length; i++) {
		for (int j = Math.max(0, (i - 1) * data.length / d.length); 
		     j <= i * data.length / d.length; j++)
		    data[j] = d[i];
		if (Math.abs(d[i]) > max)
		    max = Math.abs(d[i]);	    
	    }
	    max *= 1.1;
	    if (max == 0) max = 1;
	    repaint();
	}

	/**
	 * Setzt die Funktionswerte anhand des uebergebenen Terms, der auf dem Bereich [0, 1]
	 * ausgewertet wird
	 */
	public void setData(Term t) {
	    
	    double [] d = new double[breite];
	    for (int i = 0; i < breite; i++)
		d[i] = t.einsetzen(x, xComplex[i]).getRealPart();
	    setData(d);
	}

	public double[] getData() {

	    double [] erg = new double[data.length];
	    for (int i = 0; i < data.length; i++)
		erg[i] = data[i];

	    return erg;
	}

	public Complex[] getComplexData() {

	    Complex [] erg = new Complex[data.length];
	    for (int i = 0; i < data.length; i++)
		erg[i] = new Complex(data[i], 0);

	    return erg;
	}

	public void mouseMoved(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	private int xAlt; private double yAlt; 
	private int w; private int h;

	public void mousePressed(MouseEvent e) {

	    if (!editierbar) return;
	    // Es wird angenommen, dass sich die Groesse der Komponente 
	    // waehrend einer Mausoperation nicht aendert (wie sollte sie?)
	    w = getSize().width; h = getSize().height;

	    xAlt = Math.max(0, Math.min(data.length - 1, e.getX() * (data.length - 1) / w));
	    yAlt = max - 2 * e.getY() * max / h; 
	    mouseDragged(e);
	}

	public void mouseDragged(MouseEvent e) {

	    if (!editierbar || w == 0 || h == 0) return;

	    int x = Math.max(0, Math.min(data.length - 1, e.getX() * (data.length - 1) / w));
	    double y = max - 2 * e.getY() * max / h; 

	    if (x == xAlt) 
		data[x] = y;		
	    else {
		int x0 = Math.min(x, xAlt); double y0 = (x0 == x) ? y : yAlt;
		int x1 = Math.max(x, xAlt); double y1 = (x0 == x) ? yAlt : y; 
		for (int i = x0; i <= x1; i++)
		    data[i] = y0 + (i - x0) * (y1 - y0) / (x1 - x0);
	    }
		
	    xAlt = x; yAlt = y;
	    repaint();
	}

	public void paintComponent(Graphics g) {

	    Dimension d = getSize();
	    g.setColor(Color.white);
	    g.fillRect(0, 0, d.width, d.height);
	    g.setColor(Color.black);	    
	    g.drawLine(0, d.height / 2, d.width, d.height / 2);
	    g.drawLine(d.width / 2, 0, d.width / 2, d.height);
	    g.setColor(farbe);
	    
	    double faktor = d.height / (2 * max);
	    int xAlt = 0; int yAlt = (int) ((max - data[0]) * faktor);
	    int x = xAlt; int y = yAlt;
	    for (int i = 1; i < data.length; i++) {
		x = i * d.width / data.length;
		y = (int) ((max - data[i]) * faktor);
		g.drawLine(xAlt, yAlt, x, y);
		xAlt = x; yAlt = y;
	    }	    
	}
    }

    // #############################################################################
    // #############################################################################
    // #############################################################################

    private final class EventListener extends MouseAdapter 
	implements ActionListener, ItemListener {

	public void actionPerformed(ActionEvent e) {

	    String k = e.getActionCommand();
	    Object o = e.getSource();

	    if (k.equals("OK")) {
		closingCode = DialogPane.OK;
		dispose();
	    } else if (k.equals("Abbrechen")) {
		closingCode = DialogPane.ABBORD;
		dispose();
	    } else if (o instanceof JTextField) 
		try {
		    if (o == signalFormel)
			signal.setData(new Term(signalFormel.getText()));
		    else if (o == waveletFormel)
			wavelet.setData(new Term(waveletFormel.getText()));
		    updatePreview();
		} catch (ArithmeticException ex) {
		    GUIHelp.error(ex);
		} catch (IllegalArgumentException ex) {
		    GUIHelp.error(ex, "Syntaxfehler im Funktionsterm");
		}
	}

	public void itemStateChanged(ItemEvent e) {

	    String k = ""; boolean formel = false; boolean maus = false; boolean schnitt = false;
	    Object o = e.getSource();
	    
	    if (o == signals) {

		k = (String) signals.getSelectedItem();
		
		if (k.equals("frei (Maus)")) 
		    maus = true;
		else if (k.equals("frei (Formel)")) 
		    try {
			formel = true;
			signal.setData(new Term(signalFormel.getText()));
		    } catch (Exception ign) {
		    }
		else if (k.equals("Chirp")) 
		    signal.setData(new Term("sin(exp(1.5 * x))"));
		else if (k.equals("Zeilensummen im Bild")) {
		    if (pic == null) {
			GUIHelp.error("Es ist kein Quellbild bekannt.");
			return;
		    }			
		    double [] d = new double[pic.getHeight()];
		    for (int i = 0; i < d.length; i++) 
			for (int j = 0; j < pic.getWidth(); j++)
			    d[i] += pic.getLayer(0).getValue(i, j);
		    signal.setData(d);
		} else if (k.equals("Spaltensummen im Bild")) {
		    if (pic == null) {
			GUIHelp.error("Es ist kein Quellbild bekannt.");
			return;
		    }			
		    double [] d = new double[pic.getWidth()];
		    for (int i = 0; i < d.length; i++) 
			for (int j = 0; j < pic.getHeight(); j++)
			    d[i] += pic.getLayer(0).getValue(i, j);
		    signal.setData(d);
		} else if (k.equals("Schnitt im Bild")) {
		    if (pic == null) {
			GUIHelp.error("Es ist kein Quellbild bekannt.");
			return;
		    }
		    schnitt = true;
		}
		myJPicture.setEditable(schnitt);
		signal.setEditable(maus);
		signalFormel.setEnabled(formel);

	    } else if (o == wavelets) {
		
		k = (String) wavelets.getSelectedItem(); formel = false; maus = false;

		if (k.equals("frei (Maus)")) 
		    maus = true;
		else if (k.equals("frei (Formel)")) {
		    formel = true;
		    try {wavelet.setData(new Term(waveletFormel.getText()));
		    } catch (Exception ign) {}
		} else if (k.equals("Haar-Wavelet")) {
		    wavelet.setData(new Term("rect(0,0.5,x)-rect(0.5,1,x)"));
		} else if (k.equals("Top-Hat-Wavelet")) {
		    wavelet.setData(new Term("rect(-0.5,0.5,x)-rect(-1.5,-0.5,x)/2-" +
					     "rect(0.5,1.5,x)/2"));
		} else if (k.equals("Mexican-Hat")) {
		    wavelet.setData(new Term("(1-(2*x)^2)*exp(-(2*x)^2/2)"));
		} else if (k.equals("Morlet-Wavelet")) {
		    wavelet.setData(new Term("sin((2*x)^2/2)*exp(-(2*x)^2/2)"));
		}
		wavelet.setEditable(maus);
		waveletFormel.setEnabled(formel);
	    }
	    
	    updatePreview();
	}

	public void mouseReleased(MouseEvent e) {

	    // Die Mauseingabe eines Funktionsgraphen ist beendet worden.
	    // Wir stossen eine Berechnung der Tranformierten an.
	    if (((Graph) e.getSource()).isEditable())
		updatePreview();
	}
    }

    private static final Picture errorpic = new Picture().getPreview();

    // #############################################################################
    // #############################################################################
    // #############################################################################

    private final class MyJPicture extends JPicture 
	implements MouseListener, MouseMotionListener {

	private boolean bildVorhanden = false;
	private boolean editierbar = false;
	private int [][] schnitt = new int[][] {{0, 0}, {63, 63}};
	private final int d = 3;
	private boolean istAmEditieren = false;
	private int pkt = 0;

	public MyJPicture(Picture p) {

	    super((p == null) ? errorpic : p);
	    pic = p; bildVorhanden = (p != null);
	    setDoubleBuffered(true);
	    addMouseListener(this);
	    addMouseMotionListener(this);
	}

	public boolean isEditable() {return editierbar;}
	public void setEditable(boolean b) {

	    editierbar = b & bildVorhanden; repaint();
	    if (editierbar) updateSignal();
	}
	
	public void mouseDragged(MouseEvent e) {

	    if (!istAmEditieren) return;

	    Dimension d = getSize(); int w = d.width; int h = d.height;
	    int min = Math.min(w, h);
	    int l = (w - min) / 2; int o = (h - min) / 2;
	    schnitt[pkt][0] = Math.max(0, Math.min(min - 1, e.getX() - l)); 
	    schnitt[pkt][1] = Math.max(0, Math.min(min - 1, e.getY() - o));
	    updateSignal();
	    repaint();
	}

	public void mousePressed(MouseEvent e) {

	    Dimension dim = getSize(); int w = dim.width; int h = dim.height;
	    int min = Math.min(w, h);
	    int l = (w - min) / 2; int o = (h - min) / 2;
	    int x = e.getX() - l; int y = e.getY() - o;

	    pkt = -1; istAmEditieren = false;
	    for (int i = 0; i < 2; i++)
		if (Math.abs(x - schnitt[i][0]) <= d && Math.abs(y - schnitt[i][1]) <= d) {
		    pkt = i; istAmEditieren = true;
		}

	    mouseDragged(e);
	}

	public void mouseReleased(MouseEvent e) {

	    istAmEditieren = false; updateSignal(); updatePreview();
	}

	public void mouseMoved(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

	private void updateSignal() {

	    if (pic == null) return;

	    final int l = 200; double [] d = new double[l]; 
	    // wir rechnen die Punkte hoch und nehmen die Daten aus dem i.A.
	    // groesseren Originalbild.
	    int x0 = schnitt[0][0] * pic.getWidth() / 64;
	    int y0 = schnitt[0][1] * pic.getHeight() / 64;
	    int x1 = schnitt[1][0] * pic.getWidth() / 64;
	    int y1 = schnitt[1][1] * pic.getHeight() / 64;
	    int x; int y;
	    for (int i = 0; i < l; i++) {
		x = x0 + i * (x1 - x0) / (l - 1); y = y0 + i * (y1 - y0) / (l - 1);
		d[i] = pic.getLayer(0).getValue(x, y);
	    }

	    signal.setData(d);
	}

	public void paintComponent(Graphics g) {

	    Dimension dim = getSize(); int w = dim.width; int h = dim.height;
	    int min = Math.min(w, h);
	    int l = (w - min) / 2; int r = (w + min) / 2;
	    int o = (h - min) / 2; int u = (h + min) / 2;

	    if (!bildVorhanden) {
		g.setColor(Color.white); g.fillRect(l, o, min, min);
		g.setColor(Color.black); 
		g.drawLine(l, o, r, u); g.drawLine(l, u, r, o);
		return;
	    }	
	    
	    super.paintComponent(g);

	    if (!editierbar) return;

	    g.setColor(Color.black);
	    for (int i = 0; i < 2; i++) {
		g.drawRect(schnitt[i][0] - d + l - 1, schnitt[i][1] - d - 1 + o, 
			   2 * (d + 1), 2 * (d + 1));
		g.drawRect(schnitt[i][0] - d + l + 1, schnitt[i][1] - d + 1 + o, 
			   2 * (d - 1), 2 * (d - 1));
	    }
	    g.setColor(Color.green);
	    g.drawLine(schnitt[0][0] + l, schnitt[0][1] + o, 
		       schnitt[1][0] + l, schnitt[1][1] + o);
	    for (int i = 0; i < 2; i++) 
		g.drawRect(schnitt[i][0] - d + l, schnitt[i][1] - d + o, 2 * d, 2 * d);
	}
    }

    // #############################################################################
    // #############################################################################
    // #############################################################################

    private int closingCode = DialogPane.WINDOW_CLOSING;
	
    private Graph signal = new Graph();
    private Graph wavelet = new Graph();

    private JComboBox wavelets = new JComboBox();
    private JComboBox signals = new JComboBox();

    private JTextField waveletFormel = new JTextField("exp(-x^2/2)", 20);
    private JTextField signalFormel =  new JTextField("sin(1/x)", 20);
    private JTextField startA =        new JTextField("0.001", 5);
    private JTextField zielA =         new JTextField("1", 5);
    private JTextField startB =        new JTextField("-1", 5);
    private JTextField zielB =         new JTextField("1", 5);
    private JTextField bildgroesse =   new JTextField("128", 6);

    private Picture pic = null;
    private JPicture ergebnis = new JPicture(new Picture().getPreview());

    private MyJPicture myJPicture;

    private EventListener ev = new EventListener();

    public WaveletCenter(JFrame owner) {this(owner, null);}

    public WaveletCenter(JFrame owner, Picture pic) {

	super(owner, "kontinuierliche Wavelettransformation");

	if (pic.getNumberOfLayers() > 1)
	    pic = new Picture(new Layer[]{pic.getLayer(0)}); // nur eine Ebene

	JPanel c = new JPanel(new BorderLayout(10, 10));
	getContentPane().add(c);

	this.pic = pic;
	myJPicture = new MyJPicture((pic == null) ? null : pic.getPreview());

	JPanel signalPanel = new JPanel(new BorderLayout());
	JPanel signalUntenPanel = new JPanel(new GridLayout(0, 1, 5, 5));
	JPanel waveletPanel = new JPanel(new BorderLayout());
	JPanel waveletUntenPanel = new JPanel(new GridLayout(0, 1, 5, 5));
	JPanel vorschauPanel = new JPanel(new BorderLayout());
	JPanel originalPanel = new JPanel(new BorderLayout());
	JPanel buttonPanel = new JPanel(new GridLayout(1, 0, 10, 10));
	JPanel grenzenPanel = new JPanel(new GridLayout(0, 1, 5, 5));
	JPanel aPanel = new JPanel(new GridLayout(1, 0, 5, 5));
	JPanel bPanel = new JPanel(new GridLayout(1, 0, 5, 5));
	JPanel bildGroessePanel = new JPanel(new GridLayout(1, 0, 5, 5));
	JPanel vorschauUndGrenzen = new JPanel(new BorderLayout(10, 10));

	JButton ok = new JButton("OK");
	JButton abbruch = new JButton("Abbrechen");

	signalPanel.add("Center", signal);
	signalPanel.add("South", signalUntenPanel);

	signalUntenPanel.add(new JLabel("Signal"));
	signalUntenPanel.add(signalFormel);
	signalUntenPanel.add(signals);

	waveletPanel.add("Center", wavelet);
	waveletPanel.add("South", waveletUntenPanel);

	waveletUntenPanel.add(new JLabel("Wavelet"));
	waveletUntenPanel.add(waveletFormel);
	waveletUntenPanel.add(wavelets);

	vorschauPanel.add("West", ergebnis);
	vorschauPanel.add("South", new JLabel("Vorschau"));

	originalPanel.add("West", myJPicture);
	originalPanel.add("South", new JLabel("Original"));

	aPanel.add(startA); aPanel.add(new JLabel(" < a <")); aPanel.add(zielA);
	bPanel.add(startB); bPanel.add(new JLabel(" < b <")); bPanel.add(zielB);
	bildGroessePanel.add(new JLabel("Bildgre:"));
	bildGroessePanel.add(bildgroesse);

	grenzenPanel.add(aPanel);
	grenzenPanel.add(bPanel);
	grenzenPanel.add(bildGroessePanel);

	vorschauUndGrenzen.add("West", originalPanel);
	vorschauUndGrenzen.add("East", vorschauPanel);
	vorschauUndGrenzen.add("South", grenzenPanel);

	buttonPanel.add(ok);
	buttonPanel.add(abbruch);
	buttonPanel.add(new JPanel());

	c.add("West", signalPanel);
	c.add("Center", waveletPanel);
	c.add("East", vorschauUndGrenzen);
	c.add("South", buttonPanel);

	// Verkabelung

	c.setBorder(new EmptyBorder(10, 10, 10, 10));	
	            
	Font ff = new Font("Monospaced", Font.PLAIN, 14);
	waveletFormel.setFont(ff);
	waveletFormel.setEnabled(false);
	waveletFormel.addActionListener(ev);
	
	signalFormel.setFont(ff);
	signalFormel.setEnabled(false);
	signalFormel.addActionListener(ev);

	startA.setFont(ff); startA.addActionListener(ev);
	startB.setFont(ff); startB.addActionListener(ev);
	zielA.setFont(ff); zielA.addActionListener(ev);
	zielB.setFont(ff); zielB.addActionListener(ev);
	
	wavelet.addMouseListener(ev);
	signal.addMouseListener(ev);

	ok.addActionListener(ev); ok.setActionCommand("OK");
	abbruch.addActionListener(ev); abbruch.setActionCommand("Abbrechen");

	wavelets.addItem("frei (Maus)");
	wavelets.addItem("frei (Formel)");
	wavelets.addItem("Haar-Wavelet");
	wavelets.addItem("Top-Hat-Wavelet");
	wavelets.addItem("Mexican-Hat");
	wavelets.addItem("Morlet-Wavelet");
	wavelets.addItemListener(ev);
	wavelets.setSelectedItem("Mexican-Hat");

	signals.addItem("frei (Maus)");
	signals.addItem("frei (Formel)");
	signals.addItem("Chirp");
	signals.addItem("Zeilensummen im Bild");
	signals.addItem("Spaltensummen im Bild");
	signals.addItem("Schnitt im Bild");
	signals.addItemListener(ev);
	if (pic != null)
	    signals.setSelectedItem("Schnitt im Bild");

	updatePreview();

	// Abschlussbehandlung
	pack();
	Toolkit tk = Toolkit.getDefaultToolkit();
	Dimension screen = tk.getScreenSize();
	Dimension size = getSize();
	setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);
    }

    private static final String [] einsetzvar = new String[0]; 
    private static final Complex [] einsetzzahl = new Complex[0];

    private void updatePreview() {

	double a0 = 0.001; double a1 = 1; double b0 = -1; double b1 = 1;
	try {
	    a0 = new Term(startA.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	    a1 = new Term(zielA.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	    b0 = new Term(startB.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	    b1 = new Term(zielB.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	} catch (ArithmeticException e) {
	    GUIHelp.error(e);
	} catch (Exception e) {
	    GUIHelp.error(e, "Fehler bei den Grenzen von a und b");
	} 
	
	Picture p = getWaveletTransform(wavelet.getData(), signal.getData(), 
						  64, 64, a0, a1, b0, b1);
	ergebnis.setPictureModel(p);
	ergebnis.repaint();
    }

    /**
     * Berechnet die kontinuierliche Wavelettransformierte des signals <tt>signal</tt>
     * unter Verwendung des Wavelets <tt>wavelet</tt>. Die Felder werden aufgefasst als
     * eine Abtastung der Funktionen auf dem Intervall [0, 1], d.h. etwa 
     * <center>
     * <i>Psi</i>(0) = <tt>wavelet[0]</tt>, <i>Psi</i>(1) = <tt>wavelet[wavelet.length - 1]</tt>
     * </center>
     * Das Ergebnis wird als Layer ausgegeben, wobei die <i>a</i> entlang den Zeilen
     * und die <i>b</i> entlang den Spalten jeweils im Intervall [1, 2] variieren. 
     * Mit <tt>width</tt> und <tt>height</tt> wird die Groesse des Ergebnisfensters
     * festgelegt.
     */
    public Picture getWaveletTransform(double[] wavelet, double[] signal, 
						 int width, int height, double startA,
						 double zielA, double startB, double zielB) {

	double[][] data = new double[width][height];
	double a; double b; double erg;

	for (int x = 0; x < width; x++)
	    for (int y = 0; y < height; y++) {

		a = startA + x * (zielA - startA) / (width - 1); 
		b = startB + y * (zielB - startB) / (height - 1);

		// Analyseformel:
		// (T f)(a, b) = 1/sqrt(a) * int(psi((t - b) / a) * f(t), t=-infty..infty)
		data[x][y] = integral(wavelet, a, b, signal) / Math.sqrt(a);
	    }

	return new Picture(new Layer[]{new Layer(data)});
    }

    /**
     * Berechnet \int_{-20}^{20} g( (t - b) / a ) * f(t) dt
     */
    private double integral(double[] g, double a, double b, double [] f) {

	double start = -20; double ziel = 20;
	int step = 50; // Anzahl der Zerlegungspunkte (muss gerade sein!)

	double t = start; double d = (ziel - start) / step;
	double fertig = wert(f, t) * wert(g, (t - b) / a);

	// Nach Simpson
	int vorfaktor = 4;
	for (int i = 1; i < step; i++) { // 4 f_1 + 2 f_2 + ... + 4 f_(step-1)
	    t += d;
	    fertig += vorfaktor * wert(f, t) * wert(g, (t - b) / a);
	    vorfaktor = ((vorfaktor == 4) ? 2 : 4);
	}
	fertig += wert(f, ziel) * wert(g, (ziel - b) / a);

	return fertig / 3;
    }

    private double wert(double[] f, double x) {

	int i = (int) ((x + 5) / 10 * f.length);
	if (i < 0 || i >= f.length)
	    return 0;

	return f[i];
    }

    public Picture getResultingPicture() {

	String gr = bildgroesse.getText();
	int bg = (int) new Term(gr).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	double a0 = 0.001; double a1 = 1; double b0 = -1; double b1 = 1;
	try {
	    a0 = new Term(startA.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	    a1 = new Term(zielA.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	    b0 = new Term(startB.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	    b1 = new Term(zielB.getText()).einsetzen(einsetzvar, einsetzzahl).getRealPart();
	} catch (ArithmeticException e) {
	    GUIHelp.error(e);
	} catch (Exception e) {
	    GUIHelp.error(e, "Fehler bei den Grenzen von a und b");
	} 

	return getWaveletTransform(wavelet.getData(), signal.getData(), bg, bg, a0, a1, b0, b1);
    }

    public int getClosingCode() {return closingCode;}
}
