/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import picture.*;
import picture.gui.*;
import picture.filter.*;
import picture.math.Term;

/**
 * Dialogfenster zur Generierung von Wavelets
 */
public class WaveletPane extends DialogPane {

    // #############################################################################
    // ###  Array-Composer  ########################################################
    // #############################################################################

    private class ArrayComposer extends JPanel implements ActionListener {

	private int rechts = 1;
	private Vector rechtsV = new Vector();
	private JPanel rechtsP = new JPanel(new FlowLayout(FlowLayout.LEFT));

	private JPanel innen;
	private JScrollPane sp;

	public ArrayComposer() {

	    JButton plusRechts = holeButton("+", "+r");
	    JButton minusRechts = holeButton("-", "-r");
	    
	    JPanel buttonsRechts = new JPanel(new GridLayout(0, 1, 0, 0));
	    
	    innen = rechtsP;

	    JPanel mitte = holeEingabefeld(0);
	    rechtsV.addElement(mitte); rechtsP.add(mitte);
	    
	    sp = new JScrollPane(innen, JScrollPane.VERTICAL_SCROLLBAR_NEVER,
				 JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

	    setLayout(new BorderLayout());
	    add("Center", sp);
	    add("East", buttonsRechts);

	    buttonsRechts.add(plusRechts);  
	    buttonsRechts.add(minusRechts);
	}

	public JButton holeButton(String s, String c) {

	    JButton jb = new JButton(s); 
	    jb.setMargin(new Insets(0, 0, 0, 0));
	    jb.addActionListener(this);
	    jb.setActionCommand(c);
	    jb.setFocusPainted(false);
	    return jb;
	}

	public JPanel holeEingabefeld(int i) {return holeEingabefeld(0, i);}

	public JPanel holeEingabefeld(double d, int i) {

	    JPanel p = new JPanel(new GridLayout(0, 1));
	    JTextField tf = new JTextField("" + d, 5);
	    tf.addActionListener(this);
	    p.add(new JLabel("h[" + i + "]", JLabel.CENTER));
	    p.add(tf);
	    return p;
	}

	public void actionPerformed(ActionEvent e) {

	    if (e.getSource() instanceof JTextField) {
		neuesWavelet();
		return;
	    }

	    String k = e.getActionCommand();
	    if (k.equals("+r")) {
		JPanel p = holeEingabefeld(rechts++);
		rechtsP.add(p); rechtsV.addElement(p);
	    } else if (k.equals("-r") && rechts > 1) {
		rechtsP.remove((Component)rechtsV.elementAt(--rechts));
		rechtsV.removeElementAt(rechts);
	    } 

	    // Eins von den folgenden tut's. Wer Lust und Zeit hat, kann die anderen loeschen.
	    innen.validate(); innen.repaint(); sp.validate(); sp.repaint(); repaint();
	}

	public double [] getArray() {

	    String[] x = new String[0]; Complex[] b = new Complex[0];
	    double [] erg = new double[rechts];
	    Term t; JPanel j; JTextField jtf;
	    
	    for (int i = 0; i < rechts; i++) 
		erg[i] = getValue((JPanel) rechtsV.elementAt(i));
	    
	    return erg;
	}

	public void setArray(double [] d) {

	    if (d == null || d.length == 0)
		return;

	    while (rechts > 0) {
		rechtsP.remove((Component)rechtsV.elementAt(--rechts));
		rechtsV.removeElementAt(rechts);
	    }
	    
	    for (int i = 0; i < d.length; i++) {
		JPanel p = holeEingabefeld(d[i], rechts++);
		rechtsP.add(p); rechtsV.addElement(p);
	    }

	    repaint();
	}

	private JTextField jtextfield; private Term term;
	private String [] x = new String[0];
	private Complex [] y = new Complex[0];

	private synchronized double getValue(JPanel j) {

	    String zahl = ((JTextField) j.getComponent(1)).getText();
	    double d = 0;
	    try {
		d = new Term(zahl).einsetzen(x, y).getRealPart();
	    } catch (IllegalArgumentException e) {
		d = new Double(zahl).doubleValue();
	    }
	    return d;
	}
    }

    // #############################################################################
    // ###  Graph  #################################################################
    // #############################################################################

    private class Graph extends JComponent
	implements ChangeListener, ItemListener, FocusListener, ActionListener {

	private int daubKoeffsAlt = 0;
	private boolean ana = false;
	private boolean det = false;

	public Graph() {

	    setDoubleBuffered(true);
	}

	public void stateChanged(ChangeEvent e) {

	    Object o = e.getSource();
	    boolean changed = false;
	    int v = daubechiesSlider.getValue();

	    if (o == daubechiesSlider && v != daubKoeffsAlt && v % 2 == 0) {
		daubKoeffsAlt = v; changed = true;
	    } 

	    if (ana != analyse.isSelected()) {
		ana = analyse.isSelected(); changed = true;
	    }
	    if (det != details.isSelected()) {
		det = details.isSelected(); changed = true;
	    }

	    if (changed) 
		neuesWavelet();
	}

	public void focusGained(FocusEvent e) {}
	public void focusLost(FocusEvent e) {neuesWavelet();}
	public void itemStateChanged(ItemEvent e) {neuesWavelet();}
	public void actionPerformed(ActionEvent e) {neuesWavelet();}
	public Dimension getPreferredSize() {return new Dimension(128, 64);}

	public void paintComponent(Graphics g) {

	    Dimension d = getSize();
	    g.setColor(Color.white); g.fillRect(0, 0, d.width, d.height);

	    double max = Double.NEGATIVE_INFINITY;
	    double min = Double.POSITIVE_INFINITY;

	    for (int i = 0; i < h.length; i++) {
		if (h[i] > max) max = h[i];
		if (h[i] < min) min = h[i];
	    }

	    max = Math.abs(max + 0.5); min = Math.abs(min + 0.5);
	    max = Math.max(max, min); min = -max;

	    int yAlt = y(d.height, min, max, 0);
	    g.setColor(Color.black); // Koordinatensystem einzeichnen
	    g.drawLine(0, yAlt, d.width, yAlt);

	    // Graph
	    int xAlt = 0; int x = 0; int y = 0;
	    for (int i = 1; i <= h.length; i++) {
		x = i * d.width / (h.length + 1);
		g.setColor(Color.lightGray); g.drawLine(x, 0, x, d.height);
		y = y(d.height, min, max, h[i - 1]);
		g.setColor(Color.blue); g.drawLine(xAlt, yAlt, x, y);
		xAlt = x; yAlt = y;		
	    }
	    g.drawLine(xAlt, yAlt, d.width, y(d.height, min, max, 0));
	}

	public int y(int height, double min, double max, double y) {

	    return height - (int) (height * (y - min) / (max - min));
	}
    }

    // #############################################################################
    // ###   eigentliche Klasse  ###################################################
    // #############################################################################

    // Wavelet-Daten
    private Wavelet wavelet;
    private double [] h;

    // Swing-Komponenten
    private JComboBox modus = new JComboBox();
    private JComboBox wavelets = new JComboBox();
    private JRadioButton analyse = new JRadioButton("Analyse");
    private JRadioButton synthese = new JRadioButton("Synthese");
    private JCheckBox details = new JCheckBox("Details skalieren");
    private ArrayComposer ac = new ArrayComposer();
    private Graph graph = new Graph();
    private JSlider daubechiesSlider = new JSlider(4, 20, 4);

    // CardLayout-Mechanismus
    private CardLayout parameterCL = new CardLayout();
    private Container parameterPanel = new JPanel(parameterCL);

    /**
     * Erzeugt ein neues WaveletPane
     * @param f Mutter-Frame
     * @param pic Layer, auf das die Transformation angewandt werden soll.
     */
    public WaveletPane(JFrame f, Picture pic) throws PictureException {

	this(f, pic, Wavelet.getHaarWavelet(Wavelet.ANALYSIS));
    }
    
    /**
     * Erzeugt ein neues WaveletPane
     * @param f Mutter-Frame
     * @param pic Layer, auf das die Transformation angewandt werden soll.
     * @param wav Wavelet
     */
    public WaveletPane(JFrame f, Picture pic, Wavelet wav) throws PictureException {

	super(f, "Wavelettransformation", pic);

	// Initialisierung
	wavelet = wav;
	h = wavelet.getCoefficients();
	updater.add(wavelet);

	wavelets.addItem("Haar"); 
	wavelets.addItem("frei");	 
	wavelets.addItem("Daubechies");	 

	modus.addItem("ein Iterationsschritt");
	modus.addItem("Wegwerfen/Ergnzen von Details");
	modus.addItem("komplette Zerlegung");

	String mode = "";
	boolean ana = true;
	switch (wavelet.getMode()) {
	case Wavelet.ANALYSIS: mode = "ein Iterationsschritt"; break;
	case Wavelet.SYNTHESIS: mode = "ein Iterationsschritt"; ana = false; break;
	case Wavelet.TOTAL_ANALYSIS: mode = "komplette Zerlegung"; break;
	case Wavelet.TOTAL_SYNTHESIS: mode = "komplette Zerlegung"; ana = false; break;
	case Wavelet.ANALYSIS_WITHOUT_DETAILS: 
	    mode = "Wegwerfen/Ergnzen von Details"; 
	    details.setEnabled(false); break;
	case Wavelet.SYNTHESIS_WITHOUT_DETAILS: 
	    mode = "Wegwerfen/Ergnzen von Details"; ana = false; 
	    details.setEnabled(false); break;
	}
	modus.setSelectedItem(mode);
	details.setSelected(wavelet.isScalingDetails());

	// sieh nach, ob das angegebene Wavelet ein Haar- oder Daubechies-Wavelet ist.
	// Falls nicht, schaufele die Koeffizienten in den ArrayComposer.
	// Dies geschieht am Ende der Methode, nach Aufruf von prepareWindow(), da sonst
	// im Fall eines ueberlangen Feldes das Dialogfenster ueberbreit wird.
	boolean fillComposer = true;

	// Schluesselwort fuer das parameterPanel, das gleich angezeigt wird.
	String heinz = "diskret"; 
	switch (wavelet.getType()) {
	case Wavelet.COMMON: 
	    heinz = "diskret"; 
	    break;
	case Wavelet.HAAR: 
	    heinz = "leer"; 
	    fillComposer = false;
	    wavelets.setSelectedItem("Haar"); 
	    break;
	default: 
	    fillComposer = false; heinz = "daubechies";
	    wavelets.setSelectedItem("Daubechies");
	    daubechiesSlider.setValue(h.length);
	}
	
	// Jetzt widmen wir uns langam den Aeusserlichkeiten...

	setSourcePreview(pic); changeCursor = true;

	JPanel c = (JPanel) getContentPane();
	JPanel links = new JPanel(new BorderLayout());
	JPanel auswahl = new JPanel(new GridLayout(0, 1, 5, 5));
	JPanel auswahlLabels = new JPanel(new GridLayout(0, 1, 5, 5));
	JPanel buttons = new JPanel(new FlowLayout(FlowLayout.LEFT));

	JPanel untenLeer = new JPanel();
	JPanel daubechies = new JPanel(new GridLayout(0, 1));

	// Layout
	graph.setBorder(BorderFactory.createEtchedBorder());

	buttons.add(analyse); buttons.add(synthese); buttons.add(details);
	
	auswahlLabels.add(new JLabel("Aktion:"));	auswahl.add(modus);
	auswahlLabels.add(new JLabel("Wavelet:"));	auswahl.add(wavelets);

	links.add("West", auswahlLabels);
	links.add("East", auswahl);
	links.add("South", buttons);

	daubechies.add(new JLabel("Anzahl der Koeffizienten"));
	daubechies.add(daubechiesSlider);

	parameterPanel.add("leer", untenLeer);
	parameterPanel.add("diskret", ac);
	parameterPanel.add("daubechies", daubechies);

	c.add("East", graph);
	c.add("West", links);
	c.add("South", parameterPanel);

	// Verkabelung

	wavelets.addItemListener(graph);	daubechiesSlider.addChangeListener(graph);
	modus.addItemListener(graph);
	analyse.addItemListener(graph);
	synthese.addItemListener(graph);
	details.addItemListener(graph);

	ButtonGroup bg = new ButtonGroup();
	bg.add(analyse); bg.add(synthese);
	if (ana)
	    analyse.setSelected(true);
	else
	    synthese.setSelected(true);

	daubechiesSlider.setMinorTickSpacing(2);
	daubechiesSlider.setMajorTickSpacing(2);
	daubechiesSlider.setPaintLabels(true);
	daubechiesSlider.setSnapToTicks(true);
	daubechiesSlider.setPaintTicks(true);

	prepareWindow();
	parameterCL.show(parameterPanel, heinz);

	if (fillComposer) {
	    ac.setArray(h);
	    wavelets.setSelectedItem("frei");
	}
    }

    private void neuesWavelet() {
	
	try {
	    
	    String m = (String) modus.getSelectedItem();
	    int mode = -1;
	    boolean ana = analyse.isSelected();

	    if (m.equals("ein Iterationsschritt")) 
		mode = ana ? Wavelet.ANALYSIS : Wavelet.SYNTHESIS;
	    else if (m.equals("Wegwerfen/Ergnzen von Details"))
		mode = ana ? Wavelet.ANALYSIS_WITHOUT_DETAILS : 
		Wavelet.SYNTHESIS_WITHOUT_DETAILS;
	    else if (m.equals("komplette Zerlegung"))
		mode = ana ? Wavelet.TOTAL_ANALYSIS : Wavelet.TOTAL_SYNTHESIS;

	    String s = (String) wavelets.getSelectedItem();
	    
	    details.setEnabled((mode != Wavelet.ANALYSIS_WITHOUT_DETAILS) &&
			       (mode != Wavelet.SYNTHESIS_WITHOUT_DETAILS));

	    if (s.equals("frei")) {
		parameterCL.show(parameterPanel, "diskret");
		wavelet = new Wavelet(ac.getArray(), mode, details.isSelected());
	    } else if (s.equals("Haar")) {
		parameterCL.show(parameterPanel, "leer");
		wavelet = Wavelet.getHaarWavelet(mode, details.isSelected());
	    } else if (s.equals("Daubechies")) {
		parameterCL.show(parameterPanel, "daubechies");
		int n = daubechiesSlider.getValue();
		try {
		    wavelet = Wavelet.getDaubechiesWavelet(n, mode, details.isSelected());
		} catch (IllegalArgumentException e) {
		    GUIHelp.error(e, "Es sind nur gerade Anzahlen von Koeffizienten mglich.");
		}
	    } else 
		GUIHelp.error("Diese Option ist noch nicht\nimplementiert.");
	    
	    updater.add(wavelet); h = wavelet.getCoefficients(); graph.repaint();
	} catch (Exception e) {
	    GUIHelp.error(e);
	}
    }

    public Transformation getTransformation() {return wavelet;}

    public synchronized Picture getResultingPicture() throws PictureException {

	Picture pic = getSource();
	ProgressHandler ph = pic.getProgressHandler();

	if (wavelet == null)
	    throw new PictureException("Fehler bei der Erzeugung des Filters");

	if (ph != null) 
	    ph.setBounds(0, wavelet.getNumberOfProgressSteps(pic));

	return wavelet.of(pic);
    }
}
